SystemOrganization addCategory: #'Alt-Browser-Commands'!
SystemOrganization addCategory: #'Alt-Browser-Tests'!
SystemOrganization addCategory: #'Alt-Browser'!

ComposableModel subclass: #AltBrowser
	instanceVariableNames: 'treeModel textModel history'
	classVariableNames: 'announcer debug rootItems temporary'
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltBrowser commentStamp: '<historical>' prior: 0!
An AltBrowser is the main application.

Instance Variables
	methodModel:		<A method model>
	treeModel:		<An ABPackageList>

methodModel
	- handle the display of methods.

treeModel
	- handle navigation between packages/category/classes/protocols/methods

Class side:
	- track system announcements.!

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/18/2012 22:52'!
aCategoryRemoved: anEvent
	"Ok, a class category was removed. Not sure this works with the RPackages anymore; probably we do have to change that."

	| node r |
	node := self roots
		detect: [ :e | (r := e findClassCategory: anEvent categorySelector) notNil ] ifNone: [^self].
	r parentNode contents remove: r.
	announcer announce: (AltNodeRemoved node: r)! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/15/2012 11:10'!
aClassAdded: anEvent
	"Ok, a class was added. Find the package it belongs to (or none)."

	| node r package cat |
	package := anEvent classAdded package.
	self assert: package notNil.
	node := self roots detect: [ :e | (r := e findPackage: package) notNil ] ifNone: [ ^ self ].
	r contentsNotNil
				ifTrue: [ 
					| classNode |
					classNode := r addChild: anEvent classAdded.
					announcer announce: (AltNodeAdded node: classNode) ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/19/2012 11:28'!
aClassCommented: anEvent
	"Find the comment node and refresh it (or the class node, should be good enough)."

	| node r |
	node := self roots detect: [ :e | (r := e findClass: anEvent classCommented) notNil ].
	self aNodeToRebuilt: r! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/20/2012 22:14'!
aClassRemoved: anEvent
	"Ok, a class was removed. If this class has extensions, it should be removed from more than one place."

	| node r o |
	o := OrderedCollection new.
	node := self roots
		do: [ :e | 
			(r := e findAllClass: anEvent classRemoved) notNil
				ifTrue: [ o addAll: r ] ].
	o
		do: [ :e | self aNodeToRebuilt: e parentNode ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/18/2012 14:08'!
aClassReorganized: anEvent
	"Ok, a class was reorganised (a new protocol, for example). It seems I need to rebuilt all nodes on that class. It's not so simple than calling the new rebuilt; we also need to remove the old items or add new ones."

	| node r o |
	o := OrderedCollection new.
	node := self roots
		do: [ :e | 
			(r := e findAllClass: anEvent classReorganized) notNil
				ifTrue: [ o addAll: r ] ].
	o do: [ :each | self aNodeToRebuilt: each ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/18/2012 23:13'!
aMCWorkingCopyCreated: anEvent
	"Only case where we don't use a #findXXX: on the root items."

	"Done for the RPackage change. Need to test."

	| packageSet packageName cat node item |
	packageSet := anEvent workingCopy packageSet.
	packageName := packageSet packageName.
	item := packageSet packages size = 1
		ifTrue: [ packageSet packages first ]
		ifFalse: [ packageName -> packageSet packages asOrderedCollection ].
	cat := packageName copyUpTo: $-.
	node := self roots detect: [ :e | (e isKindOf: ABPackageCategoryNode) and: [ e item key = cat ] ] ifNone: [ nil ].
	node notNil
		ifTrue: [ 
			| packageNode |
			node item value add: item.
			packageNode := node addChild: item.
			announcer announce: (AltNodeAdded node: packageNode) ]
		ifFalse: [ 
			node := packageSet packages size = 1
				ifTrue: [ ABPackageNode with: item ]
				ifFalse: [ ABPackageCategoryNode with: item ].
			self roots add: node.
			announcer announce: (AltNodeAdded node: node) ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/19/2012 11:42'!
aMCWorkingCopyDeleted: anEvent
	"Depending on the MCPackage, this may appear as a package category or as a RPackage."

	| aPackageSet package cat node r |
	aPackageSet := anEvent workingCopy packageSet.
	self roots contents
		detect: [ :e | (node := e findPackage: aPackageSet packages first) notNil ]
		ifNone: [ ^ self ].
	aPackageSet packages size > 1
		ifTrue: [ node := node parentNode ].
	node parentNode notNil
		ifTrue: [ node parentNode removeChildItem: node ]
		ifFalse: [ self roots remove: node ].
	announcer announce: (AltNodeRemoved node: r)! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/21/2012 11:26'!
aMCWorkingCopyModified: anEvent
	"Ok, should force a change in look for the relevant object in the trees."

	| packageSet node |
	packageSet := anEvent package.
	self roots detect: [ :e | (node := e findPackageSet: packageSet) notNil ] ifNone: [ nil ].
	node notNil
		ifTrue: [ self aNodeToUpdate: node ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/22/2012 11:54'!
aMethodAdded: anEvent
	"Ok, a method was added. Things to search for: extensions!! If the method is an extension, update that part as well (and at the package extension level, since it may be necessary to add the class node as well). Use the findAllMethodParents: for that."

	"Problem : if there isn't any method yet, then the current method category will be replaced by the 'as yet unclassified' category, and the update won't work as expected."

	| mCNodes |
	mCNodes := OrderedCollection new.
	self roots
		do: [ :root | 
			| res |
			res := root findAllClass: anEvent methodClass.
			res notNil
				ifTrue: [ mCNodes addAll: res ] ].
	mCNodes
		do: [ :each | 
			| pNode |
			pNode := each findMethodCategory: anEvent protocol in: anEvent methodClass.
			(each contents detect: [ :i | i item = 'no messages' ] ifNone: [ nil ]) isNil
				ifFalse: [ self aNodeToRebuilt: each ]
				ifTrue: [ self aNodeToRebuilt: pNode ] ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/22/2012 11:23'!
aMethodRecategorized: anEvent
	"Ok, a method protocol has changed. Don't forget the method may be in multiple places (extensions) but this may be handled by other announcements ."

	"TOCHECK: if a protocol is removed or added because of this event, suppose that the system will send a ClassReorganized event as well."

	| mCNodes |
	mCNodes := OrderedCollection new.
	self roots
		do: [ :root | 
			| res |
			res := root findAllClass: anEvent methodClass.
			res notNil
				ifTrue: [ mCNodes addAll: res ] ].
	mCNodes
		do: [ :each | 
			| pNode pNode2 |
			pNode := each findMethodCategory: anEvent oldProtocol in: anEvent methodClass.
			self aNodeToRebuilt: pNode.
			pNode2 := each findMethodCategory: anEvent newProtocol in: anEvent methodClass.
			pNode2 ~= pNode
				ifTrue: [ self aNodeToRebuilt: pNode ] ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/22/2012 12:01'!
aMethodRemoved: anEvent
	"Ok, a method was removed. Don't forget the method may be in multiple places."

	"Change this code; search for places where the method is visible (extensions and protocols)."

	"Bug: when removing a method, the method having already been removed from the system makes the search for the parent fail... :palm:"

	| mCNodes |
	mCNodes := OrderedCollection new.
	self roots
		do: [ :root | 
			| res |
			res := root findAllClass: anEvent methodClass.
			res notNil
				ifTrue: [ mCNodes addAll: res ] ].
	mCNodes
		do: [ :each | 
			| pNode |
			pNode := each findMethodCategory: anEvent protocol in: anEvent methodClass.
			(anEvent protocol = #'as yet unclassified' and: [ pNode isNil ])
				ifTrue: [ self aNodeToRebuilt: each ]
				ifFalse: [ self aNodeToRebuilt: pNode ] ]! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/19/2012 15:19'!
aNodeToRebuilt: r
	"Ok. The clever updating is done and it works rather nicely. Will help simplify the extensions to IndentingListItemMorph."

	r isNil ifTrue: [ ^ self ].
	r rebuildContents.
	announcer announce: (AltNodeRebuilt node: r)! !

!AltBrowser class methodsFor: 'updating' stamp: 'ThierryGoubier 9/21/2012 09:55'!
aNodeToUpdate: r
	"Update the node without rebuilding it. Good for PackageSet which may change its label based on its contents."

	r isNil ifTrue: [ ^ self ].
	announcer announce: (AltNodeUpdated node: r)! !

!AltBrowser class methodsFor: 'accessing' stamp: 'ThierryGoubier 8/28/2012 14:03'!
announcer
	announcer isNil
		ifTrue: [ announcer := Announcer new ].
	^announcer! !

!AltBrowser class methodsFor: 'debug' stamp: 'ThierryGoubier 9/19/2012 10:42'!
debug
	debug isNil
		ifTrue: [ debug := false ].
	^ debug! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 8/28/2012 21:49'!
defaultPackageCategoriesNames
	"As it says: categories for packages who don't use the - convention. But also the place where we may start organizing nicely the system to have a better structure. Do a reset of the AltBrowser to see the changes in the opened browsers."
	
	"AltBrowser reset."
	
	^ #('System'
		#('GUI' 'Graphics' 'Morphic' 'Balloon' 'FreeType' 'Polymorph' 'Spec' 'UIManager' 'UI' 'GroupManagerUI' 'Growl' 'Keymapping' 'FamFam')
		#('Core' 'Kernel' 'Collections' 'Text' 'Compiler' 'EmergencyEvaluator' 'Regex' 'Multilingual' 'Announcements' 'Transcript' 'Traits')
		#('Development' 'AST' 'Refactoring' 'Tools' 'Nautilus' 'NonInteractiveTranscript' 'SUnit' 'Tests' 'Shout' 'SmaCC' 'NECompletion' 'NOCompletion')
		#('Networking' 'Network' 'Zodiac' 'Zinc' 'Compression')
		#('Packages' 'PackageInfo' 'Fuel' 'Monticello' 'CodeImport' 'Gofer' 'Configuration' 'RPackage' 'Ring' 'CI' 'GroupManager')
		#('2.0' 'Compatibility' 'Deprecated' 'RecentSubmissions' 'FixUnderscores' 'ScriptLoader20')
		
	)! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 9/24/2012 11:27'!
defaultSpec
	<spec>
	
	^ {#ComposableSpec.
			#add:. {
				{#model. #treeModel}.
				#layout:.
				{#FrameLayout. #fractions:offsets:. (0 @ 0 corner: 0.3 @ 1). (0 @ 0 corner: 0 @ 0)}
			}.
			#add:. {
				{#model. #textModel}.
				#layout:.
				{#FrameLayout. #fractions:offsets:. (0.3 @ 0 corner: 1 @ 1). (0 @ 0 corner: 0 @ 0)}
			}
		}! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 7/26/2012 10:42'!
example
	"AltBrowser example"
	
	AltBrowser new openWithSpec! !

!AltBrowser class methodsFor: 'instance creation' stamp: 'ThierryGoubier 9/8/2012 11:19'!
fullOnClass: aClass
	"Open on a class."

	| s |
	self roots detect: [ :each | (s := each findClass: aClass) notNil ].
	self open selectPathForAbstractItem: s! !

!AltBrowser class methodsFor: 'instance creation' stamp: 'ThierryGoubier 9/10/2012 16:58'!
fullOnClass: aClass selector: aSelector
	"Open on a class."

	| s |
	aSelector isNil ifTrue: [ ^ self fullOnClass: aClass]. 
	self roots detect: [ :each | (s := each findMethod: aSelector in: aClass) notNil ].
	self open selectPathForAbstractItem: s! !

!AltBrowser class methodsFor: 'instance creation' stamp: 'ThierryGoubier 9/7/2012 15:59'!
fullOnEnvironment: anEnvironment
" I am not environment aware "
	
	self open! !

!AltBrowser class methodsFor: 'instance creation' stamp: 'ThierryGoubier 9/11/2012 10:06'!
fullOnMethods: aListOfCM
	"Given a list of compiled methods, open, and preselect all of them."

	| s l n |
	n := self open.
	(l := aListOfCM collect: [ :each | each compiledMethod ])
		do: [ :aCompiledMethod | 
			self roots detect: [ :each | (s := each findMethod: aCompiledMethod selector in: aCompiledMethod methodClass) notNil ].
			aCompiledMethod == l last
				ifTrue: [ n selectPathForAbstractItem: s ]
				ifFalse: [ 
					| k |
					n treeModel getRoots
						detect: [ :r | 
							| it |
							(k := r forceFindItem: s) notNil ].
					self assert: k notNil.
					self assert: k class == ABTreeNodeModel.
					n history add: k ] ]! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 9/19/2012 22:22'!
getItemsFor: aCategoryDefinition from: aListOfPackages
	"Remove elements from aListOfPackages to bring them accordingly to a CategoryDefinition. Returns the top-level object. If this definition is an array, go recursively through it."

	aCategoryDefinition isArray
		ifTrue: [ 
			| i |
			i := aCategoryDefinition first -> OrderedCollection new.
			aCategoryDefinition allButFirst do: [ :each | i value add: (self getItemsFor: each from: aListOfPackages) ].
			^ i ]
		ifFalse: [ 
			| i |
			i := aCategoryDefinition -> OrderedCollection new.
			(aListOfPackages
				select: [ :e | 
					e packageName size >= aCategoryDefinition size
						and: [ (e packageName copyFrom: 1 to: aCategoryDefinition size) = aCategoryDefinition ] ])
				do: [ :e | 
					aListOfPackages remove: e.
					i value add: e ].
			^ i ]! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 9/19/2012 22:24'!
getRootItems
	"Build the organisation out of some default classification and collect all remaining packages from the list of MCWorkingCopy in the system."

	"There is still some bugs in the packages introduced with the first implementation of RPackage and they show when we build this list."

	| l o |
	l := MCWorkingCopy allManagers asOrderedCollection.
	o := OrderedCollection new.
	self defaultPackageCategoriesNames
		do: [ :w | 
			| i |
			i := self getItemsFor: w from: l.
			o add: i ].
	[ l notEmpty ]
		whileTrue: [ 
			| p w i |
			p := l first.
			w := p packageName copyUpTo: $-.
			i := w -> (OrderedCollection with: p).
			o add: i.
			l remove: p.
			(l select: [ :e | (e packageName copyUpTo: $-) = w ])
				do: [ :e | 
					l remove: e.
					i value add: e ] ].
	^ o
		collect: [ :e | 
			e value size ~= 1
				ifTrue: [ ABPackageCategoryNode with: e ]
				ifFalse: [ ABPackageSetNode with: e value first ] ]! !

!AltBrowser class methodsFor: 'initialization' stamp: 'ThierryGoubier 9/9/2012 18:49'!
initialize
	announcer := Announcer new.
	rootItems := nil.
	self registerOnSystemAnnouncements.
	self registerInAppRegistry
! !

!AltBrowser class methodsFor: 'user interface' stamp: 'ThierryGoubier 8/29/2012 14:54'!
menuCommandOn: aBuilder 
	<worldMenu>
	
	(aBuilder item: #'Alt Browser')
			parent: #Tools;
			action: [ self open ];
			icon: UITheme current smallSystemBrowserIcon;
			help: 'An Alternative System Browser'.! !

!AltBrowser class methodsFor: 'instance creation' stamp: 'ThierryGoubier 9/7/2012 16:00'!
newOnCategory: aCategory
	"Who is calling that ? Category don't have that much sense anymore."
	
	^ self open! !

!AltBrowser class methodsFor: 'instance creation' stamp: 'ThierryGoubier 9/7/2012 16:01'!
newOnClass: aClass
	^ self fullOnClass: aClass! !

!AltBrowser class methodsFor: 'instance creation' stamp: 'ThierryGoubier 9/7/2012 16:03'!
newOnClass: aClass selector: aSelector
	^ self fullOnClass: aClass selector: aSelector! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 9/19/2012 13:53'!
open
	"AltBrowser open. Make sure that, in the absence of selections, we set the tree keymap."

	| n |
	(n := AltBrowser new) openWithSpec.
	n window model: n.
	n treeModel updateKeymap.
	^ n! !

!AltBrowser class methodsFor: 'instance creation' stamp: 'ThierryGoubier 9/11/2012 10:32'!
openMessageList: messageList name: aString autoSelect: aSelector
	" Tool registry compatibility. Disregard the name and consider autoselect as allways true."
	
	"In fact, due to the speed of scanning and opening each path in the tree, this is far too slow. Maybe a solution, but less than adequate, could be to only open path on the very first element, but load the history with the others : this solution works, but is not as good as the direct one. Left in case we find something else."

	^ self fullOnMethods: messageList! !

!AltBrowser class methodsFor: 'initialization' stamp: 'ThierryGoubier 9/9/2012 18:47'!
registerInAppRegistry
	"Register the receiver in the SystemBrowser AppRegistry"

	SystemBrowser register: self! !

!AltBrowser class methodsFor: 'initialization' stamp: 'ThierryGoubier 9/22/2012 11:26'!
registerOnSystemAnnouncements
	"Global registration to update the list."

	"self registerOnSystemAnnouncements."

	SystemAnnouncer uniqueInstance unsubscribe: self.
	SystemAnnouncer uniqueInstance weak
		on: MCWorkingCopyCreated send: #aMCWorkingCopyCreated: to: self;
		on: MCWorkingCopyDeleted send: #aMCWorkingCopyDeleted: to: self;
		on: MCPackageModified send: #aMCWorkingCopyModified: to: self;
		on: CategoryRemoved send: #aCategoryRemoved: to: self;
		on: MethodAdded send: #aMethodAdded: to: self;
		on: MethodRemoved send: #aMethodRemoved: to: self;
		on: MethodRecategorized send: #aMethodRecategorized:: to: self;
		on: ClassAdded send: #aClassAdded: to: self;
		on: ClassRemoved send: #aClassRemoved: to: self;
		on: ClassReorganized send: #aClassReorganized: to: self;
		on: ClassCommented send: #aClassCommented: to: self! !

!AltBrowser class methodsFor: 'debug' stamp: 'ThierryGoubier 9/19/2012 10:42'!
removeDebug
	debug := false! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 9/8/2012 11:11'!
reset
	"reset the rootItems. Should propagate to all instances as well."
	
	"AltBrowser reset"

	rootItems := self getRootItems.
	announcer announce: AltRootsReset new.! !

!AltBrowser class methodsFor: 'accessing' stamp: 'ThierryGoubier 9/8/2012 11:13'!
roots
	rootItems isNil
		ifTrue: [ rootItems := self getRootItems ].
	^ rootItems! !

!AltBrowser class methodsFor: 'debug' stamp: 'ThierryGoubier 9/19/2012 10:42'!
setDebug
	debug := true! !

!AltBrowser class methodsFor: 'user interface' stamp: 'ThierryGoubier 9/18/2012 14:52'!
taskbarIcon
	^ UITheme current smallSystemBrowserIcon! !

!AltBrowser class methodsFor: 'accessing' stamp: 'ThierryGoubier 9/9/2012 22:23'!
temporary
	^ temporary ! !

!AltBrowser class methodsFor: 'accessing' stamp: 'ThierryGoubier 9/9/2012 23:26'!
temporary: aValue
	temporary := aValue! !

!AltBrowser class methodsFor: 'spec' stamp: 'ThierryGoubier 7/15/2012 22:44'!
title
	^ 'Alt Browser'! !

!AltBrowser class methodsFor: 'private' stamp: 'ThierryGoubier 7/26/2012 16:23'!
updateAll
	"Entry point for updating the whole tree if needed."
	
	self reset! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 9/21/2012 09:56'!
aMethodUpdated: anEvent
	"A method has changed. Check if this is the one we are selecting; if yes, update it."

	| sItem |
	self selectedItem isNil ifTrue: [ ^self ].
	sItem := self selectedItem item.
	self assert: (sItem isKindOf: ABAbstractNode).
	(sItem isKindOf: ABMethodNode)
		ifFalse: [ ^ self ].
	(sItem sourceClass == anEvent methodClass and: [ sItem item == anEvent oldMethod selector ])
		ifTrue: [ textModel text: self selectedItem sourceCode ]! !

!AltBrowser methodsFor: 'menu' stamp: 'ThierryGoubier 9/19/2012 13:49'!
addModelItemsToWindowMenu: aMenu
	aMenu addLine.
	aMenu add: 'Set debug' target: self class action: #setDebug.
	aMenu add: 'Remove debug' target: self class action: #removeDebug.
	aMenu add: 'Reset' target: self class action: #reset.
	aMenu add: 'Initialize' target: self class action: #initialize! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 8/28/2012 14:07'!
addNode: aNodeAdded
	"Announcement : A node has been added."

	treeModel rootsAdd: aNodeAdded node! !

!AltBrowser methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:22'!
buildMenu: aMenu on: aTarget
	treeModel selectedItem notNil
		ifTrue: [ treeModel selectedItem buildTextMenu: aMenu on: aTarget with: self ].
	^ aMenu! !

!AltBrowser methodsFor: 'commands' stamp: 'ThierryGoubier 7/31/2012 15:57'!
buildTreeMenu: aMenu on: aTarget
	"Ask the selection for builidng a menu, otherwise create a default one."
	
	treeModel selectedItem notNil
		ifTrue: [ treeModel selectedItem buildTreeMenu: aMenu on: aTarget with: self ]
		ifFalse: [ 
			| t |
			t := false.
			(Pragma allNamed: #treeAreaCommand from: self class to: AltBrowser)
				do: [ :e | 
					t
						ifTrue: [ aMenu addLine ]
						ifFalse: [ t := true ].
					(self perform: e selector) do: [ :c | (c on: aTarget for: self) addItemToMenu: aMenu ] ] ].
	^ aMenu! !

!AltBrowser methodsFor: 'menu' stamp: 'ThierryGoubier 9/16/2012 08:15'!
defaultCommands
	<treeAreaCommand>
	^ {ABCreatePackageCategoryCommand.
	ABFindClassCommand}! !

!AltBrowser methodsFor: 'initialization' stamp: 'ThierryGoubier 9/22/2012 11:13'!
delete
	treeModel notNil
		ifTrue: [ treeModel roots: #() ].
	super delete.
	self release! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 9/4/2012 22:08'!
findNodeForItem: anItem
	| result |
	treeModel getRoots detect:
		[:rootItem |
		(result := rootItem findItem: anItem) notNil ].
	^ result! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 9/4/2012 22:05'!
forceFindNodeForItem: anItem
	| result |
	treeModel getRoots detect:
		[:rootItem |
		(result := rootItem forceFindItem: anItem) notNil ].
	^ result! !

!AltBrowser methodsFor: 'private' stamp: 'ThierryGoubier 9/8/2012 11:16'!
getRootItems
	^ self class roots collect: [ :e | ABTreeNodeModel with: e model: e ]! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 9/19/2012 10:41'!
history
	history isNil
		ifTrue: [ 
			history := NavigationHistory new.
			history maxSize: 50 ].
	^ history! !

!AltBrowser methodsFor: 'protocol' stamp: 'ThierryGoubier 8/7/2012 20:31'!
initialExtent
	^ 700 @ 500! !

!AltBrowser methodsFor: 'initialization' stamp: 'ThierryGoubier 9/21/2012 16:15'!
initializePresenter
	"The models are created, it means I can register on events."
	
	treeModel roots: self getRootItems.
	treeModel childrenBlock: [ :it | it contents ].
	treeModel displayBlock: [ :it | it item asText ].
	treeModel
		whenSelectedItemChanged: [ :sel |
			treeModel updateKeymap.
			textModel updateKeymap.
			self updateTitle.
			sel isNil
				ifTrue: [ 
					textModel text: ''.
					textModel behavior: nil.
					textModel aboutToStyle: false ]
				ifFalse: [ 
					textModel text: sel sourceCode.
					textModel behavior: sel sourceClass.
					textModel aboutToStyle: sel isStyled.
					self history add: sel ] ].
	textModel menu: [ :m :t | self buildMenu: m on: t ].
	treeModel menu: [:m :t | self buildTreeMenu: m on: t].
	
	treeModel keymapHolder: [:kmDispatcher | self updateTreeKeymap: kmDispatcher].
	textModel keymapHolder: [:kmDispatcher | self updateTextKeymap: kmDispatcher].
	treeModel updateKeymap.
	textModel updateKeymap.
	
	self class announcer weak on: AltRootsReset send: #reset to: self.
	self class announcer weak on: AltNodeRebuilt send: #rebuiltNode: to: self.
	self class announcer weak on: AltNodeRemoved send: #removeNode: to: self.
	self class announcer weak on: AltNodeAdded send: #addNode: to: self.
	self class announcer weak on: AltNodeUpdated send: #updateNode: to: self.

	SystemAnnouncer uniqueInstance weak on: MethodModified send: #aMethodUpdated: to: self.! !

!AltBrowser methodsFor: 'initialization' stamp: 'ThierryGoubier 9/17/2012 21:43'!
initializeWidgets
	self instantiateModels: #(#treeModel #ABTreeModel #textModel #ABTextModel).
	textModel aboutToStyle: true
! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 9/21/2012 09:57'!
rebuiltNode: aNodeRebuilt
	"Announcement : a node has been rebuilt."
	
	treeModel rebuildNode: aNodeRebuilt node! !

!AltBrowser methodsFor: 'initialization' stamp: 'ThierryGoubier 9/21/2012 07:06'!
release
	"There is something about releasing models."

	treeModel
		ifNotNil: [ 
			treeModel initialize.
			treeModel := nil ].
	textModel
		ifNotNil: [ 
			textModel initialize.
			textModel := nil ].
	self class announcer unsubscribe: self.
	SystemAnnouncer uniqueInstance unsubscribe: self.
	super release! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 9/4/2012 22:03'!
removeNode: aNodeRemoved
	"Announcement : A node in the tree has been removed. Update the tree."

	| aNode |
	aNode := aNodeRemoved node.
	treeModel rootsRemove: aNode! !

!AltBrowser methodsFor: 'private' stamp: 'ThierryGoubier 9/5/2012 22:55'!
reset
	treeModel isNil
		ifFalse: [ 
			treeModel roots: self getRootItems.
			textModel text: ''.
			textModel behavior: nil.
			textModel aboutToStyle: false ].
	history := nil! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 9/6/2012 23:18'!
selectPathForAbstractItem: anItem
	"Stronger than the item version, will force an opening of the ABTreeModelNodes."

	| found aClass nPath tmPath aNode |
	nPath := OrderedCollection new.
	aNode := anItem.
	[ aNode notNil ]
		whileTrue: [ 
			nPath add: aNode.
			aNode := aNode parentNode ].
	tmPath := OrderedCollection with: (treeModel getRoots detect: [ :r | r item == nPath last ]).
	nPath removeLast.
	nPath reverseDo: [ :n | tmPath add: (tmPath last forceFindItem: n) ].
	treeModel widget selectPath: tmPath in: (treeModel widget scroller submorphs at: 1 ifAbsent: [ ^ self ])! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 9/4/2012 22:05'!
selectPathForItem: anItem
	"Select an object by finding it's path and all. This makes sure the tree display is entirely open for that."

	| found aClass nPath aNode |
	nPath := OrderedCollection new.
	aNode := anItem.
	[ aNode notNil ]
		whileTrue: [ 
			nPath addFirst: aNode.
			aNode := aNode parentNode ].
	treeModel widget selectPath: nPath in: (treeModel widget scroller submorphs at: 1 ifAbsent: [ ^ self ])! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 7/26/2012 10:45'!
selectedItem
	^ treeModel selectedItem! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 9/4/2012 16:41'!
selectedItem: anItem
	"May not be working at all."

	| tMorph |
	treeModel widget notNil
		ifTrue: [ tMorph := (treeModel widget scroller submorphs at: 1) findItem: anItem ].
	tMorph notNil
		ifTrue: [ treeModel widget setSelectedMorph: tMorph ]
		ifFalse: [ anItem explore ]! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 21:56'!
taskbarIcon
	^ UITheme current smallSystemBrowserIcon! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 7/16/2012 00:35'!
textModel
	^textModel! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 9/21/2012 16:16'!
title
	self selectedItem notNil
		ifTrue: [ ^ 'Alt: ' , self selectedItem title ]
		ifFalse: [ ^ 'Alt Browser' ]! !

!AltBrowser methodsFor: 'accessing' stamp: 'ThierryGoubier 7/15/2012 22:41'!
treeModel
	^treeModel! !

!AltBrowser methodsFor: 'updating' stamp: 'ThierryGoubier 9/21/2012 11:26'!
updateNode: aNodeUpdated
	"Announcement : a node has been updated."
	
	treeModel updateNode: aNodeUpdated node! !

!AltBrowser methodsFor: 'shortcuts' stamp: 'ThierryGoubier 9/14/2012 22:13'!
updateTextKeymap: aKMDispatcher
	"Update the text keymap. Reset the dispatcher to avoid the default shortcuts."

	aKMDispatcher reset.
	aKMDispatcher attachCategory: #Morph.	"navigation focus shortcuts."
	self selectedItem notNil
		ifTrue: [ self selectedItem item buildTextShortcutsOn: aKMDispatcher with: self ]! !

!AltBrowser methodsFor: 'shortcuts' stamp: 'ThierryGoubier 9/14/2012 22:10'!
updateTreeKeymap: aKMDispatcher
	"Update the tree keymap. Change the key dispatcher if needed. Target for now is the PluggableTreeMorph, we will have to see if this is the correct one."

	aKMDispatcher resetPerInstanceTarget.
	self selectedItem notNil
		ifTrue: [ self selectedItem item buildTreeShortcutsOn: aKMDispatcher with: self ]
		ifFalse: [ 
			(Pragma allNamed: #treeAreaCommand from: self class to: AltBrowser)
				do: [ :e | (self perform: e selector) do: [ :c | (c on: self treeModel for: self) buildShortcut: aKMDispatcher ] ] ]! !

!WindowModel methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 8/1/2012 21:55'!
taskbarIcon
	^ self model taskbarIcon! !

TreeModel subclass: #ABTreeModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABTreeModel commentStamp: '<historical>' prior: 0!
An ABTreeModel is an extension of TreeModel to handle drag and drop.

My problem with the drag and drop is that I loose the wrapper object (and therefore the logic) which is needed when the drag starts. I can :
* add another level of wrapper object below the list objects.
* have only one type of list wrapper object and my own wrappers.
* subclass the PluggableTreeMorph.!

!ABTreeModel class methodsFor: 'specs' stamp: 'ThierryGoubier 9/17/2012 07:01'!
defaultSpec
	<spec>
	
	^ {#TreeSpec.
	    		#model:.							#model.
			#getLabelSelector:.				#wrapItem:.
			#getRootsSelector:.				#getRoots.
			#getChildrenSelector:.				#getChildrenFor:.
			#setSelectedSelector:.			#setSelectedItem:.
			#color:.							Color white.
			#getMenuSelector:.				#getMenu:.
			#hResizing:. 						#spaceFill.
			#vResizing:. 						#spaceFill.
			#dropItemSelector:.				#dropItem:on:.
			#dragItemSelector:.				#dragItem:.
			#wantsDropSelector:.				#wantsDropItem:.
			#getHelpSelector:.				#selectedItemName:.}! !

!ABTreeModel methodsFor: 'drag and drop' stamp: 'ThierryGoubier 7/31/2012 11:07'!
dragItem: anItem
	^ anItem! !

!ABTreeModel methodsFor: 'drag and drop' stamp: 'ThierryGoubier 8/1/2012 21:22'!
dropItem: anItem on: aTarget
	"This is called. Yes, really."

	aTarget notNil
		ifTrue: [ aTarget dropItem: anItem ]! !

!ABTreeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/1/2012 22:38'!
rebuildNode: aNode
	| r result |
	rootsHolder contents detect: [ :rootItem | (result := rootItem findItem: aNode) notNil ] ifNone: [ ^ self ].
	result notNil
		ifTrue: [ 
			result rebuildContents.
			r := (self widget scroller submorphs at: 1) findItem: result.
			r notNil
				ifTrue: [ r rebuildChildren ] ]! !

!ABTreeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/18/2012 14:01'!
rootsAdd: aNode
	"Algorithm: look for the parent of the node. If this parent node is found, then add a node equivalent to the contents if they do exist."

	| pNode |
	(pNode := aNode parentNode) isNil
		ifTrue: [ 
			self assert: (aNode isKindOf: ABTreeNodeModel) not.
			rootsHolder add: (ABTreeNodeModel with: aNode model: aNode) beforeIndex: (AltBrowser roots indexOf: aNode).
			(self widget scroller submorphs at: 1) checkInvariant ]
		ifFalse: [ 
			rootsHolder contents
				do: [ :e | 
					| r |
					(r := e findItem: pNode) notNil
						ifTrue: [ 
							| n wNode |
							r contentsNotNil
								ifTrue: [ wNode := r addChild: aNode ].
							(n := (self widget scroller submorphs at: 1) findItem: r) notNil
								ifTrue: [ n rebuildChildren ].
							(self widget scroller submorphs at: 1) checkInvariant.
							^ self ] ] ]! !

!ABTreeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/18/2012 15:34'!
rootsRemove: aNode
	"Beware : a node is not an instance of ABTreeNodeModel"

	self assert: aNode isTreeNodeModel not.
	rootsHolder contents
		do: [ :e | 
			| r |
			(r := e findItem: aNode) notNil
				ifTrue: [ 
					r == e
						ifTrue: [ rootsHolder remove: r ifAbsent: [ nil ] ]
						ifFalse: [ 
							| n |
							r parentNode contents remove: r.
							(n := (self widget scroller submorphs at: 1) findItem: r) notNil
								ifTrue: [ n rebuildChildren ].
							(self widget scroller submorphs at: 1) checkInvariant ] ] ]! !

!ABTreeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/18/2012 18:30'!
selectedItemName: anItem

	^ anItem item asString! !

!ABTreeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/21/2012 13:31'!
updateNode: aNode
	| r result |
	rootsHolder contents detect: [ :rootItem | (result := rootItem findItem: aNode) notNil ] ifNone: [ ^ self ].
	result notNil
		ifTrue: [ 
			r := (self widget scroller submorphs at: 1) findItem: result.
			r notNil
				ifTrue: [ r contents: r complexContents asString ] ]! !

!ABTreeModel methodsFor: 'drag and drop' stamp: 'ThierryGoubier 7/31/2012 10:49'!
wantsDropItem: anItem
	"TODO: Filter out to avoid drag and drop of different objet types. Here, we don't know what the target may be so filtering is limited."
	
	^ true
	! !

Announcement subclass: #AltNodeAnnouncement
	instanceVariableNames: 'node'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltNodeAnnouncement commentStamp: '<historical>' prior: 0!
The abstract node announcement.

Beware, unexpected behavior!! Announcement matching for actions matches also subclasses (what a strange idea?).

Buggy previous code: AltNodeRemoved has AltNodeAdded as superclass; on AltNodeRemoved both AltNodeAdded and AltNodeRemoved would match and trigger actions.
!

AltNodeAnnouncement subclass: #AltNodeAdded
	instanceVariableNames: 'index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltNodeAdded commentStamp: '<historical>' prior: 0!
This announce an added node in the tree.
!

!AltNodeAnnouncement class methodsFor: 'instance creation' stamp: 'ThierryGoubier 8/29/2012 22:33'!
node: aNode
	^ self new node: aNode! !

!AltNodeAnnouncement methodsFor: 'accessing' stamp: 'ThierryGoubier 8/29/2012 22:28'!
node
	^ node! !

!AltNodeAnnouncement methodsFor: 'accessing' stamp: 'ThierryGoubier 8/29/2012 22:28'!
node: anObject
	node := anObject! !

AltNodeAnnouncement subclass: #AltNodeRebuilt
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltNodeRebuilt commentStamp: '<historical>' prior: 0!
This announcement mark a node rebuilt.!

AltNodeAnnouncement subclass: #AltNodeRemoved
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltNodeRemoved commentStamp: '<historical>' prior: 0!
This class mark the removal of a node.!

AltNodeAnnouncement subclass: #AltNodeUpdated
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

Announcement subclass: #AltRootsReset
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltRootsReset commentStamp: '<historical>' prior: 0!
This announcement is for resetting all roots from the AltBrowser class side.!

CustomHelp subclass: #AltBrowserHelp
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltBrowserHelp commentStamp: '<historical>' prior: 0!
This is the help for the AltBrowser. Tries to hold some documentation about what is going on.
!

!AltBrowserHelp class methodsFor: 'accessing' stamp: 'ThierryGoubier 9/1/2012 18:46'!
bookName
	^ 'The Alternative Browser Help'! !

!AltBrowserHelp class methodsFor: 'pages' stamp: 'ThierryGoubier 9/1/2012 18:52'!
introduction
	^ HelpTopic
		title: 'Introduction'
		contents:
'The Alternative System Browser


The AltBrowser, for short, is a replacement for the standard or improved system browser. Its aims are to present a powerfull, yet simple, view of your Smalltalk Image to explore and program with.'
		! !

!AltBrowserHelp class methodsFor: 'accessing' stamp: 'ThierryGoubier 9/1/2012 18:53'!
pages
	^ #(introduction)! !

!AltBrowserHelp class methodsFor: 'pages' stamp: 'ThierryGoubier 9/6/2012 21:44'!
status
	^ HelpTopic
		title: 'Status'
		contents:
'Status is : moving forward quite well. However, it has a tendency to expose bugs in RPackage which makes things the more difficult.'
		! !

TextModel subclass: #ABTextModel
	instanceVariableNames: 'menuBlockHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABTextModel commentStamp: '<historical>' prior: 0!
An ABTextModel is an extension to TextModel allowing the override of the menu.
!

!ABTextModel class methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 9/13/2012 13:06'!
defaultSpec
	<spec>
	
	^ SpecLayout text
			send: #on:text:accept:readSelection:menu: withArguments: #(model getText accept:notifying: readSelection getMenu:);
			send: #classOrMetaClass: withArguments: {#(model behavior)};
			send: #enabled: withArguments: {#(model enabled)};
			send: #vSpaceFill;
			send: #hSpaceFill;
			yourself! !

!ABTextModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 22:39'!
getMenu: aMenu

	menuBlockHolder contents cull: aMenu cull: self.
	^ aMenu! !

!ABTextModel methodsFor: 'initialization' stamp: 'ThierryGoubier 7/17/2012 22:40'!
initialize
	"Initialization code for TreeModel"

	super initialize.

	menuBlockHolder := [] asValueHolder.! !

!ABTextModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 22:39'!
menu: aBlock
	"Set the block used to generate the tree menu"

	menuBlockHolder contents: aBlock! !

KMDispatcher subclass: #AltKMDispatcher
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltKMDispatcher commentStamp: '<historical>' prior: 0!
Here only to remove a bad behavior in KMDispatcher. See how the standard KMDispatcher is replaced by this one in AltBrowser>>updateTextKeymaps.

Also remove the staticTargets targets, so that we may have a text editor without any legacy shortcuts.
!

!AltKMDispatcher methodsFor: 'dispatching' stamp: 'ThierryGoubier 9/10/2012 23:30'!
keymapObservers
	| o |
	(o := OrderedCollection with: self perInstanceTarget)
		addAll: self targets.
	^ o! !

MorphTreeNodeModel subclass: #ABTreeNodeModel
	instanceVariableNames: 'contents'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABTreeNodeModel commentStamp: '<historical>' prior: 0!
An ABTreeNodeModel represent a node for the tree. It reroutes most of it's request to the underlying item.

It caches its contents however.!

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/3/2012 21:58'!
addChild: anItem
	"Add a child, but do it according to it's position."

	| d newNode |
	d := self childrenItems indexOf: anItem.
	d < 1 ifTrue: [^ self].
	contents add: (newNode := self childNodeFromItem: anItem) beforeIndex: d.
	^ newNode! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/25/2012 16:53'!
asText
	^ item asText! !

!ABTreeNodeModel methodsFor: 'querying' stamp: 'ThierryGoubier 7/25/2012 23:23'!
buildTextMenu: aMenu on: aTarget with: aRequestor
	^ self item buildTextMenu: aMenu on: aTarget with: aRequestor! !

!ABTreeNodeModel methodsFor: 'querying' stamp: 'ThierryGoubier 7/30/2012 16:54'!
buildTreeMenu: aMenu on: aTarget with: aRequestor
	^ self item buildTreeMenu: aMenu on: aTarget with: aRequestor! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:33'!
childrenItems
	^ item contents! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:26'!
contents
	contents isNil
		ifTrue: [ contents := super contents ].
	^ contents! !

!ABTreeNodeModel methodsFor: 'testing' stamp: 'ThierryGoubier 7/27/2012 21:42'!
contentsNotNil
	^ contents notNil! !

!ABTreeNodeModel methodsFor: 'querying' stamp: 'ThierryGoubier 7/26/2012 23:16'!
findItem: anItem
	"searching for an item, returns a node."

	self item == anItem
		ifTrue: [ ^ self ].
	contents notNil
		ifTrue: [ 
			contents
				do: [ :e | 
					| s |
					(s := e findItem: anItem) notNil
						ifTrue: [ ^ s ] ] ].
	^ nil! !

!ABTreeNodeModel methodsFor: 'querying' stamp: 'ThierryGoubier 8/31/2012 17:27'!
forceFindItem: anItem
	"searching for an item, returns a node."

	self item == anItem
		ifTrue: [ ^ self ].
	self contents
		do: [ :e | 
			| s |
			(s := e forceFindItem: anItem) notNil
				ifTrue: [ ^ s ] ].
	^ nil! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 23:24'!
isStyled
	^ item isStyled! !

!ABTreeNodeModel methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 14:44'!
isTreeNodeModel
	^ true! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/4/2012 12:45'!
item
	^ item! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 8/5/2012 09:10'!
rebuildChildren
	"Force a rebuild of the children, including on the item."
	
	item rebuildContents.
	contents := nil! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 8/30/2012 16:29'!
rebuildContents
	"Force a rebuild of the contents."
	
	contents := nil! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:27'!
sourceClass
	^ item sourceClass! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:27'!
sourceCode
	^ item sourceCode! !

!ABTreeNodeModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/21/2012 16:15'!
title
	^ item title! !

!ABTreeNodeModel methodsFor: 'querying' stamp: 'ThierryGoubier 7/31/2012 10:35'!
wantsDroppedObject: anotherItem
	"So far, this is unused. The wrapper never connect to that here. What is needed is to customize the PluggableTreeMorph, which is the model behind all elements of the tree."

	^ self item wantsDroppedObject: anotherItem! !

MorphTreeModel subclass: #AltBrowserModel
	instanceVariableNames: 'source tree history window treeMorph textMorph'
	classVariableNames: 'announcer debug rootItems temporary'
	poolDictionaries: ''
	category: 'Alt-Browser'!

!AltBrowserModel commentStamp: '<historical>' prior: 0!
An AltBrowserModel represent the base for a Morph-based GUI. It takes from the Jejak JViewer.

No need for tree in there ?

There is probably a need for text model handling; not sure our source thing will work.!

!AltBrowserModel class methodsFor: 'accessing' stamp: 'ThierryGoubier 9/25/2012 16:52'!
announcer
	^ AltBrowser announcer! !

!AltBrowserModel class methodsFor: 'instance creation' stamp: 'ThierryGoubier 9/25/2012 13:53'!
open
	^ self new open! !

!AltBrowserModel methodsFor: 'updating' stamp: 'ThierryGoubier 9/25/2012 15:01'!
aMethodUpdated: anEvent
	"A method has changed. Check if this is the one we are selecting; if yes, update it."

	| sItem |
	self selectedNode isNil
		ifTrue: [ ^ self ].
	sItem := self selectedNode item.
	self assert: (sItem isKindOf: ABAbstractNode).
	(sItem isKindOf: ABMethodNode)
		ifFalse: [ ^ self ].
	(sItem sourceClass == anEvent methodClass and: [ sItem item == anEvent oldMethod selector ])
		ifTrue: [ source value: self selectedNode sourceCode ]! !

!AltBrowserModel methodsFor: 'updating' stamp: 'ThierryGoubier 9/25/2012 15:08'!
aNodeAdded: aNode
	"Algorithm: look for the parent of the node. If this parent node is found, then add a node equivalent to the contents if they do exist."

	| pNode |
	(pNode := aNode parentNode) isNil
		ifTrue: [ 
			self assert: (aNode isKindOf: ABTreeNodeModel) not.
			tree roots add: (ABTreeNodeModel with: aNode model: aNode) beforeIndex: (AltBrowser roots indexOf: aNode).
			(treeMorph scroller submorphs at: 1) checkInvariant ]
		ifFalse: [ 
			tree roots
				do: [ :e | 
					| r |
					(r := e findItem: pNode) notNil
						ifTrue: [ 
							| n wNode |
							r contentsNotNil
								ifTrue: [ wNode := r addChild: aNode ].
							(n := (treeMorph scroller submorphs at: 1) findItem: r) notNil
								ifTrue: [ n rebuildChildren ].
							(treeMorph scroller submorphs at: 1) checkInvariant.
							^ self ] ] ]! !

!AltBrowserModel methodsFor: 'updating' stamp: 'ThierryGoubier 9/25/2012 15:35'!
aNodeRebuilt: aNode
	| r result |
	tree roots detect: [ :rootItem | (result := rootItem findItem: aNode) notNil ] ifNone: [ ^ self ].
	result notNil
		ifTrue: [ 
			result rebuildContents.
			r := (treeMorph scroller submorphs at: 1) findItem: result.
			r notNil
				ifTrue: [ r rebuildChildren ] ]! !

!AltBrowserModel methodsFor: 'updating' stamp: 'ThierryGoubier 9/25/2012 15:22'!
aNodeRemoved: aNode
	"Beware : a node is not an instance of ABTreeNodeModel"

	self assert: aNode isTreeNodeModel not.
	tree roots
		do: [ :e | 
			| r |
			(r := e findItem: aNode) notNil
				ifTrue: [ 
					r == e
						ifTrue: [ tree roots remove: r ifAbsent: [ nil ] ]
						ifFalse: [ 
							| n |
							r parentNode contents remove: r.
							(n := (treeMorph scroller submorphs at: 1) findItem: r) notNil
								ifTrue: [ n rebuildChildren ].
							(treeMorph scroller submorphs at: 1) checkInvariant ] ] ]! !

!AltBrowserModel methodsFor: 'updating' stamp: 'ThierryGoubier 9/25/2012 15:23'!
aNodeUpdated: aNode
	| r result |
	tree roots detect: [ :rootItem | (result := rootItem findItem: aNode) notNil ] ifNone: [ ^ self ].
	result notNil
		ifTrue: [ 
			r := (treeMorph scroller submorphs at: 1) findItem: result.
			r notNil
				ifTrue: [ r contents: r complexContents asString ] ]! !

!AltBrowserModel methodsFor: 'menus' stamp: 'ThierryGoubier 9/25/2012 16:35'!
addModelItemsToWindowMenu: aMenu
	aMenu addLine.
	aMenu add: 'Set debug' target: self class action: #setDebug.
	aMenu add: 'Remove debug' target: self class action: #removeDebug.
	aMenu add: 'Reset' target: self class action: #reset.
	aMenu add: 'Initialize' target: self class action: #initialize! !

!AltBrowserModel methodsFor: 'menus' stamp: 'ThierryGoubier 9/25/2012 16:36'!
buildMenu: aMenu on: aTarget
	self selectedNode notNil
		ifTrue: [ self selectedNode buildTextMenu: aMenu on: aTarget with: self ].
	^ aMenu! !

!AltBrowserModel methodsFor: 'menus' stamp: 'ThierryGoubier 9/25/2012 16:37'!
buildTreeMenu: aMenu on: aTarget
	"Ask the selection for builidng a menu, otherwise create a default one."

	self selectedNode notNil
		ifTrue: [ self selectedNode selectedItem buildTreeMenu: aMenu on: aTarget with: self ]
		ifFalse: [ 
			| t |
			t := false.
			(Pragma allNamed: #treeAreaCommand from: self class to: self class)
				do: [ :e | 
					t
						ifTrue: [ aMenu addLine ]
						ifFalse: [ t := true ].
					(self perform: e selector) do: [ :c | (c on: aTarget for: self) addItemToMenu: aMenu ] ] ].
	^ aMenu! !

!AltBrowserModel methodsFor: 'menus' stamp: 'ThierryGoubier 9/25/2012 16:35'!
defaultCommands
	<treeAreaCommand>
	^ {ABCreatePackageCategoryCommand.
	ABFindClassCommand}! !

!AltBrowserModel methodsFor: 'user interface' stamp: 'ThierryGoubier 9/25/2012 16:22'!
defaultTreeMorph
	^ super defaultTreeMorph
		doubleClickSelector: #doubleClick;
		nodeStringGetter: #asText;
		getMenuSelector: #menu:shifted:! !

!AltBrowserModel methodsFor: 'initialize-release' stamp: 'ThierryGoubier 9/25/2012 14:30'!
delete
	tree notNil
		ifTrue: [ tree roots: #() ].
	self release! !

!AltBrowserModel methodsFor: 'drag and drop' stamp: 'ThierryGoubier 9/25/2012 16:32'!
dragItem: anItem
	^ anItem! !

!AltBrowserModel methodsFor: 'drag and drop' stamp: 'ThierryGoubier 9/25/2012 16:33'!
dropItem: anItem on: aTarget
	"This is called. Yes, really."

	aTarget notNil
		ifTrue: [ aTarget dropItem: anItem ]! !

!AltBrowserModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/25/2012 14:12'!
findNodeForItem: anItem
	| result |
	tree getRoots detect:
		[:rootItem |
		(result := rootItem findItem: anItem) notNil ].
	^ result! !

!AltBrowserModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/25/2012 14:13'!
forceFindNodeForItem: anItem
	| result |
	tree getRoots detect:
		[:rootItem |
		(result := rootItem forceFindItem: anItem) notNil ].
	^ result! !

!AltBrowserModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/25/2012 14:12'!
history
	history isNil
		ifTrue: [ 
			history := NavigationHistory new.
			history maxSize: 50 ].
	^ history! !

!AltBrowserModel methodsFor: 'initialize-release' stamp: 'ThierryGoubier 9/25/2012 16:40'!
initialize
	"Link to the AltBrowser class for most updates."

	self class announcer weak on: AltRootsReset send: #reset to: self.
	self class announcer weak on: AltNodeRebuilt send: #aNodeRebuilt: to: self.
	self class announcer weak on: AltNodeRemoved send: #aNodeRemoved: to: self.
	self class announcer weak on: AltNodeAdded send: #aNodeAdded: to: self.
	self class announcer weak on: AltNodeUpdated send: #aNodeUpdated: to: self.
	SystemAnnouncer uniqueInstance weak on: MethodModified send: #aMethodUpdated: to: self! !

!AltBrowserModel methodsFor: 'user interface' stamp: 'ThierryGoubier 9/25/2012 14:07'!
open
	"Open the window, as simple as that."
	
	window := self theme newWindowIn: World for: self title: self title.
	treeMorph := self defaultTreeMorph.
	treeMorph buildContents.
	window
		addMorph: treeMorph
		fullFrame: (LayoutFrame fractions: (0 @ 0 corner: 0.3 @ 1) offsets: (0 @ 0 corner: 0 @ 0)).
	textMorph := PluggableTextMorph on: self source text: #contents accept: nil.
	window
		addMorph: textMorph
		fullFrame: (LayoutFrame fractions: (0.3 @ 0 corner: 1 @ 1) offsets: (0 @ 0 corner: 0 @ 0)).
	window minimumExtent: 500 @ 500.
	window open! !

!AltBrowserModel methodsFor: 'initialize-release' stamp: 'ThierryGoubier 9/25/2012 14:29'!
release
	self class announcer unsubscribe: self.
	SystemAnnouncer uniqueInstance unsubscribe: self.
	super release! !

!AltBrowserModel methodsFor: 'updating' stamp: 'ThierryGoubier 9/25/2012 16:52'!
rootItems
	^ AltBrowser roots! !

!AltBrowserModel methodsFor: 'updating' stamp: 'ThierryGoubier 9/25/2012 14:47'!
rootNodeClassFromItem: anItem
	^ ABTreeNodeModel! !

!AltBrowserModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/25/2012 14:14'!
selectPathForAbstractItem: anItem
	"Stronger than the item version, will force an opening of the ABTreeModelNodes."

	| found aClass nPath tmPath aNode |
	nPath := OrderedCollection new.
	aNode := anItem.
	[ aNode notNil ]
		whileTrue: [ 
			nPath add: aNode.
			aNode := aNode parentNode ].
	tmPath := OrderedCollection with: (tree getRoots detect: [ :r | r item == nPath last ]).
	nPath removeLast.
	nPath reverseDo: [ :n | tmPath add: (tmPath last forceFindItem: n) ].
	treeMorph selectPath: tmPath in: (treeMorph scroller submorphs at: 1 ifAbsent: [ ^ self ])! !

!AltBrowserModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/25/2012 14:14'!
selectPathForItem: anItem
	"Select an object by finding it's path and all. This makes sure the tree display is entirely open for that."

	| found aClass nPath aNode |
	nPath := OrderedCollection new.
	aNode := anItem.
	[ aNode notNil ]
		whileTrue: [ 
			nPath addFirst: aNode.
			aNode := aNode parentNode ].
	treeMorph selectPath: nPath in: (treeMorph scroller submorphs at: 1 ifAbsent: [ ^ self ])! !

!AltBrowserModel methodsFor: 'updating' stamp: 'ThierryGoubier 9/25/2012 17:00'!
selection: anItem
	"It goes directly here."

	self updateTextKeymap: textMorph kmDispatcher.
	self updateTreeKeymap: treeMorph kmDispatcher.
	self updateTitle.
	super selection: anItem.
	anItem isNil
		ifTrue: [ 
			self source value: ''	"textModel behavior: nil.
					textModel aboutToStyle: false" ]
		ifFalse: [ 
			self source value: anItem sourceCode.	"textModel behavior: sel sourceClass.
					textModel aboutToStyle: sel isStyled."
			self history add: anItem ]! !

!AltBrowserModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/25/2012 14:49'!
setRoots: roots
	tree roots: roots! !

!AltBrowserModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/25/2012 16:58'!
source
	^ source isNil
		ifTrue: [ source := ValueHolder new contents: String new ]
		ifFalse: [ source ]! !

!AltBrowserModel methodsFor: 'shortcuts' stamp: 'ThierryGoubier 9/25/2012 14:59'!
targetShortcutMorph
	"Get the real, lowest level morph we need to touch the key dispatch for. What a mess!!"

	^ textMorph submorphs first submorphs first! !

!AltBrowserModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/25/2012 14:27'!
taskbarIcon
	^ UITheme current smallSystemBrowserIcon! !

!AltBrowserModel methodsFor: 'accessing' stamp: 'ThierryGoubier 9/25/2012 14:27'!
title
	self selectedNode notNil
		ifTrue: [ ^ 'Alt: ' , self selectedNode title ]
		ifFalse: [ ^ 'Alt Browser' ]! !

!AltBrowserModel methodsFor: 'shortcuts' stamp: 'ThierryGoubier 9/25/2012 14:57'!
updateTextKeymap: aKMDispatcher
	"Update the text keymap. Reset the dispatcher to avoid the default shortcuts."

	aKMDispatcher reset.
	aKMDispatcher attachCategory: #MorphFocusCtrlNavigation.
	self selectedNode notNil
		ifTrue: [ self selectedNode item buildTextShortcutsOn: aKMDispatcher with: self ]! !

!AltBrowserModel methodsFor: 'shortcuts' stamp: 'ThierryGoubier 9/25/2012 14:58'!
updateTreeKeymap: aKMDispatcher
	"Update the tree keymap."

	aKMDispatcher resetPerInstanceTarget.
	self selectedNode notNil
		ifTrue: [ self selectedNode item buildTreeShortcutsOn: aKMDispatcher with: self ]
		ifFalse: [ 
			(Pragma allNamed: #treeAreaCommand from: self class to: self class)
				do: [ :e | (self perform: e selector) do: [ :c | (c on: self tree for: self) buildShortcut: aKMDispatcher ] ] ]! !

!AltBrowserModel methodsFor: 'drag and drop' stamp: 'ThierryGoubier 9/25/2012 16:33'!
wantsDropItem: anItem
	"TODO: Filter out to avoid drag and drop of different objet types. Here, we don't know what the target may be so filtering is limited."
	
	^ true
	! !

TestCase subclass: #AltBrowserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Tests'!

!AltBrowserTest commentStamp: '<historical>' prior: 0!
Testing for the AltBrowser is here.!

!AltBrowserTest methodsFor: 'tests' stamp: 'ThierryGoubier 8/26/2012 23:57'!
testRootItems
	self assert: (AltBrowser getRootItems notEmpty)! !

!AltBrowserTest methodsFor: 'tests' stamp: 'ThierryGoubier 8/27/2012 00:06'!
testSpawn2RootItems
	AltBrowser getRootItems do: [:e | e contents do: [:f | f contents]]! !

!AltBrowserTest methodsFor: 'tests' stamp: 'ThierryGoubier 9/19/2012 11:16'!
testSpawn3RootItems
	AltBrowser getRootItems do: [ :e | e contents do: [ :f | f contents do: [ :g | g contents ] ] ]! !

!AltBrowserTest methodsFor: 'tests' stamp: 'ThierryGoubier 8/27/2012 00:04'!
testSpawnRootItems
	AltBrowser getRootItems do: [:e | e contents]! !

!IndentingListItemMorph methodsFor: '*alt-browser' stamp: 'ThierryGoubier 9/9/2012 15:07'!
checkInvariant
	"Check that this morph is correct regarding it's abstract contents."

	"We check that siblings are correctly aligned and have the right index."

	self
		withSiblingsDo: [ :each | 
			each assert: (each firstChild isNil or: [ each complexContents contents notNil ]).
			each complexContents item parentNode notNil
				ifTrue: [ 
					each
						assert:
							((each nextSibling isNil and: [ each complexContents item = each complexContents item parentNode contents last ])
								or: [ each nextSibling notNil ]).
					each nextSibling notNil
						ifTrue: [ 
							each
								assert:
									((each complexContents item parentNode contents indexOf: each complexContents item) + 1
										= (each nextSibling complexContents item parentNode contents indexOf: each nextSibling complexContents item)
										and: [ each complexContents item parentNode == each nextSibling complexContents item parentNode ]) ] ] ].
	self
		withSiblingsDo: [ :each | 
			each firstChild notNil
				ifTrue: [ each firstChild checkInvariant ] ]! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 9/9/2012 23:19'!
findExpanded
	"Get all expanded items among children recursively."

	| currentlyExpanded |
	currentlyExpanded := OrderedCollection new.
	firstChild notNil
		ifTrue: [ firstChild withSiblingsDo: [ :aNode | currentlyExpanded addAll: aNode findExpanded ] ].
	self isExpanded
		ifTrue: [ currentlyExpanded add: self ].
	^ currentlyExpanded! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/27/2012 14:35'!
findItem: aNode
	| found |
	self
		withSiblingsDo: [ :each | 
			each complexContents item == aNode
				ifTrue: [ ^ each ] ].
	self
		withSiblingsDo: [ :each | 
			each firstChild notNil
				ifTrue: [ 
					found := each firstChild findItem: aNode.
					found notNil
						ifTrue: [ ^ found ] ] ].
	^ nil! !

!IndentingListItemMorph methodsFor: '*alt-browser' stamp: 'ThierryGoubier 9/18/2012 13:59'!
rebuildChildren
	"Force a rebuild of the childrens. May be triggered by going from no children to has children."

	self recursiveRefreshExpandedState.
	container adjustSubmorphPositions! !

!IndentingListItemMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 9/9/2012 23:19'!
recursiveRefreshExpandedState
	"Trying to do a better refreshExpandedState which remembers expanded children."

	| toDelete c final currentlyExpanded |
	toDelete := OrderedCollection new.
	currentlyExpanded := OrderedCollection new.
	firstChild
		ifNotNil: [ 
			firstChild
				withSiblingsDo: [ :aNode | 
					currentlyExpanded addAll: aNode findExpanded.
					aNode recursiveAddTo: toDelete ] ].
	container noteRemovalOfAll: toDelete.
	(isExpanded and: [ complexContents hasContents ])
		ifFalse: [ ^ self changed ].
	(c := complexContents contents) isEmpty
		ifTrue: [ ^ self changed ].
	final := OrderedCollection new.
	self addChildrenForList: container addingTo: final withExpandedItems: currentlyExpanded.
	container scroller addAllMorphs: final after: self! !

Object subclass: #ABAbstractNode
	instanceVariableNames: 'item contents parentNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABAbstractNode commentStamp: '<historical>' prior: 0!
An ABAbstractNode is the abstract type of all nodes used by the AltBrowser.!

!ABAbstractNode class methodsFor: 'instance creation' stamp: 'ThierryGoubier 7/19/2012 22:36'!
with: anItem
	^ self new item: anItem! !

!ABAbstractNode methodsFor: 'actions' stamp: 'ThierryGoubier 9/9/2012 15:06'!
accept: aText notifying: aController
	"When the text editor associated has accepted, do that..."

	"That is, nothing by default."
	
	^ nil

	! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/3/2012 18:18'!
addChild: anItem
	"Add a child, but do it according to it's position."

	| d newItemNode |
	d := self childrenItems indexOf: anItem.
	contents add: (newItemNode := self childNodeFromItem: anItem) beforeIndex: d.
	^ newItemNode! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/30/2012 00:00'!
addChildItem: anItem
	"Add a child item. Remember to effectively add both at the item level if necessary and at the contents level."

	| d |
	self childrenItems isEmpty ifTrue: [^self contents add: anItem].
	d := self childrenItems indexOf: anItem item.
	^ contents add: anItem beforeIndex: d! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 21:38'!
addChildItem: anItem beforeIndex: index
	"Add a child, but do it according to it's position."

	anItem parentNode: self.
	^ contents add: anItem beforeIndex: index! !

!ABAbstractNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/19/2012 22:43'!
asString
	^ item asString! !

!ABAbstractNode methodsFor: 'printing' stamp: 'ThierryGoubier 9/18/2012 13:16'!
asText
	^ self asString! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 8/5/2012 23:05'!
buildTextMenu: aMenu on: aTarget with: aRequestor
	| t |
	t := false.
	(Pragma allNamed: #textAreaCommand from: self class to: ABAbstractNode)
		do: [ :e | 
			t
				ifTrue: [ aMenu addLine ]
				ifFalse: [ t := true ].
			(self perform: e selector) do: [ :c | (c on: aTarget for: aRequestor) addItemToMenu: aMenu ] ]! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/5/2012 16:59'!
buildTextShortcutsOn: aKMDispatcher with: aRequestor
	"This is an attempt at handling shortcuts... Which works, with the help of a custom KMDispatcher."

	(Pragma allNamed: #textAreaCommand from: self class to: ABAbstractNode)
		do: [ :e | 
			(self perform: e selector)
				do: [ :c | 
					| command |
					command := c on: aRequestor textModel for: aRequestor.
					command buildShortcut: aKMDispatcher ] ]! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/30/2012 16:46'!
buildTreeMenu: aMenu on: aTarget with: aRequestor
	| t |
	t := false.
	(Pragma allNamed: #treeAreaCommand from: self class to: ABAbstractNode)
		do: [ :e | 
			t
				ifTrue: [ aMenu addLine ]
				ifFalse: [ t := true ].
			(self perform: e selector) do: [ :c | (c on: aTarget for: aRequestor) addItemToMenu: aMenu ] ]! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/11/2012 14:13'!
buildTreeShortcutsOn: aMorph with: aRequestor
	"This is an attempt at handling shortcuts... Which works, with the help of a custom KMDispatcher."

	(Pragma allNamed: #treeAreaCommand from: self class to: ABAbstractNode)
		do: [ :e | 
			(self perform: e selector)
				do: [ :c | 
					| command |
					command := c on: aRequestor treeModel for: aRequestor.
					command isEnabled
						ifTrue: [ command buildShortcut: aMorph ] ] ]! !

!ABAbstractNode methodsFor: 'private' stamp: 'ThierryGoubier 7/19/2012 22:32'!
childNodeClassFromItem: anItem
	^ self class! !

!ABAbstractNode methodsFor: 'private' stamp: 'ThierryGoubier 7/27/2012 21:34'!
childNodeFromItem: anItem
	^ ((self childNodeClassFromItem: anItem) with: anItem)
		parentNode: self;
		yourself! !

!ABAbstractNode methodsFor: 'private' stamp: 'ThierryGoubier 7/19/2012 22:34'!
childrenItems
	^ #()! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:18'!
clipboardCommands
	<textAreaCommand>
	^ {ABCutTextCommand.
	ABCopyTextCommand.
	ABPasteTextCommand}! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 9/9/2012 23:37'!
clipboardTreeCommands
	<treeAreaCommand>
	^ {ABCutCommand.
		ABPasteCommand}! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/3/2012 18:16'!
contents
	contents isNil
		ifTrue: [ contents := (self childrenItems collect: [ :i | self childNodeFromItem: i ]) asOrderedCollection ].
	^ contents! !

!ABAbstractNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/27/2012 21:47'!
contentsNotNil
	^ contents notNil! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 8/5/2012 09:07'!
debugTreeCommands
	<treeAreaCommand>
	^ {ABExploreItemCommand. ABRebuildItemCommand }! !

!ABAbstractNode methodsFor: 'drag and drop' stamp: 'ThierryGoubier 9/9/2012 23:33'!
dropItem: anItem
	"By default, do nothing."
	
	^ nil! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:18'!
executeCommands
	<textAreaCommand>
	^ {ABDoItTextCommand.
	ABPrintItTextCommand.
	ABInspectItTextCommand.
	ABExploreItTextCommand.
	ABDebugItTextCommand}! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 10:08'!
findAllClass: aClass
	"Return the node if the item matches. By default, search among the kids."
	
	"Make it lazy : do not trigger contents generation."

	| o res |
	contents notNil
		ifTrue: [ 
			self contents
				do: [ :each | 
					(res := each findAllClass: aClass) notNil
						ifTrue: [ 
							o isNil
								ifTrue: [ o := res ]
								ifFalse: [ o addAll: res ] ] ] ].
	^ o! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 10:09'!
findAllMethodParents: aMethodSelector forClass: aClass
	"Return all nodes if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	"Make it lazy : do not trigger contents generation."

	| o res |
	contents notNil
		ifTrue: [ 
			self contents
				do: [ :each | 
					(res := each findAllMethodParents: aMethodSelector forClass: aClass) notNil
						ifTrue: [ 
							o isNil
								ifTrue: [ o := res ]
								ifFalse: [ o addAll: res ] ] ] ].
	^ o! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 18:03'!
findClass: aClass
	"Return the node if the item matches. By default, search among the kids."

	"Not lazy : force contents generation."

	| found res |
	self contents
		do: [ :each | 
			(res := each findClass: aClass) notNil
				ifTrue: [ found := res ] ].
	^ found! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 10:10'!
findClassCategory: aCat
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	"Make it lazy : do not trigger contents generation."

	| res |
	contents notNil
		ifTrue: [ 
			self contents
				do: [ :each | 
					(res := each findClassCategory: aCat) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 10:11'!
findClassComment: aClass
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."
	
	"Make it lazy : do not trigger contents generation."

	| res |
	contents notNil
		ifTrue: [ 
			self contents
				do: [ :each | 
					(res := each findClassComment: aClass) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 10:11'!
findMethod: aSelector in: aClass
	"Return the node if the item matches. By default, search among the kids."

	"Make it lazy : do not trigger contents generation."

	| res |
	contents notNil
		ifTrue: [ 
			self contents
				do: [ :each | 
					(res := each findMethod: aSelector in: aClass) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 10:12'!
findMethodCategory: aMethodCategory in: aClass
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	"Make it lazy : do not trigger contents generation."

	| res |
	contents notNil
		ifTrue: [ 
			self contents
				do: [ :each | 
					(res := each findMethodCategory: aMethodCategory in: aClass) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 22:57'!
findPackage: aRPackage
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	"Not lazy : triggers contents generation."

	| res |
	self contents
		do: [ :each | 
			(res := each findPackage: aRPackage) notNil
				ifTrue: [ ^ res ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 18:04'!
findPackageSet: aRPackageSetOrMCWCOrMCPackage
	"Return the node if the item matches. By default, search among the kids."

	| res |
	self contents
		do: [ :each | 
			(res := each findPackageSet: aRPackageSetOrMCWCOrMCPackage) notNil
				ifTrue: [ ^ res ] ].
	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/30/2012 14:01'!
indexOfChild: anItem
	^ self childrenItems indexOf: anItem! !

!ABAbstractNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/19/2012 23:20'!
isStyled
	"Should return true if the text is to be styled (default smalltalk code)."
	
	^ false! !

!ABAbstractNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 14:44'!
isTreeNodeModel
	^ false! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
item
	^ item! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
item: anObject
	item := anObject! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
parentNode
	^ parentNode! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/19/2012 22:36'!
parentNode: anObject
	parentNode := anObject! !

!ABAbstractNode methodsFor: 'printing' stamp: 'ThierryGoubier 8/30/2012 07:04'!
printOn: aStream
	super printOn: aStream.
	aStream nextPut: $(.
	aStream nextPutAll: self asString.
	aStream nextPut: $)! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/18/2012 14:14'!
rebuildContents
	"Force a rebuild of the contents, to either : change the layout of the children items, or just because the contents has become out of sync with the children items."

	"Optimize that to reuse existing objects as much as possible."

	contents notNil
		ifTrue: [ 
			contents := (self childrenItems
				collect: [ :i | 
					| n |
					n := contents detect: [ :e | e item == i ] ifNone: [ nil ].
					n isNil
						ifTrue: [ self childNodeFromItem: i ]
						ifFalse: [ n ] ]) asOrderedCollection ]! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/3/2012 18:17'!
removeChildItem: anItem
	"Remove a child item. Some subclasses may do things with the item, or translate this into an operation."

	self contents remove: anItem! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:19'!
saveCommands
	<textAreaCommand>
	^ {ABAcceptTextCommand.
	ABCancelTextCommand}! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 9/4/2012 16:25'!
searchTreeCommands
	<treeAreaCommand>
	^ {ABFindClassCommand. ABPreviousCommand. ABNextCommand }! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/19/2012 23:20'!
sourceClass
	"Should return a class if appropriate. The class is to provide the context for compilation, evaluation or styling."

	^ nil! !

!ABAbstractNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/19/2012 23:19'!
sourceCode
	"Should return a source code or a description if appropriate."
	
	^ ''! !

!ABAbstractNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/21/2012 16:12'!
title
	^ ''! !

!ABAbstractNode methodsFor: 'commands' stamp: 'ThierryGoubier 7/25/2012 23:19'!
undoCommands
	<textAreaCommand>
	^ {ABUndoTextCommand.
	ABRedoTextCommand}! !

ABAbstractNode subclass: #ABClassCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassCategoryNode commentStamp: '<historical>' prior: 0!
An ABClassCategoryNode is for class categories.
!

!ABClassCategoryNode class methodsFor: 'ordering' stamp: 'ThierryGoubier 9/11/2012 17:12'!
orderClasses: aClassList
	"Order classes according to their superclasses."

	| r addClassBlock c |
	addClassBlock := [ :fromList :toList :aClass | 
	(toList includes: aClass)
		ifFalse: [ 
			(fromList includes: aClass superclass)
				ifTrue: [ 
					(toList includes: aClass superclass)
						ifFalse: [ addClassBlock value: fromList value: toList value: aClass superclass ].
					toList add: aClass afterIndex: (toList indexOf: aClass superclass) ]
				ifFalse: [ toList addFirst: aClass ] ] ].
	r := OrderedCollection new.
	c := SortedCollection sortBlock: [:a :b | a name > b name].
	c addAll: aClassList.
	c do: [ :eachClass | addClassBlock value: aClassList value: r value: eachClass ].
	^ r! !

!ABClassCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 7/31/2012 22:28'!
childNodeClassFromItem: anItem
	(anItem isBehavior or: [ anItem isTrait ])
		ifTrue: [ ^ ABClassNode ]
		ifFalse: [ ^ self class ]! !

!ABClassCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 9/10/2012 13:24'!
childrenItems
	"Extract from the organisation the various classes. Ok. Our upper level should try to built a hierarchy with the category name, if applicable (via various upTo: $-). This code is ready for that (introduce an additional hierarchy)."

	"We use Smalltalk organisation for that, not Rpackage stuff, and they may diverge (they do)."

	| c o r addClassBlock |
	c := Smalltalk organization categoriesMatching: self item.
	c size > 2
		ifTrue: [ ^ c ]
		ifFalse: [ ^ self class orderClasses: (Smalltalk organization classesInCategory: c first) ]! !

!ABClassCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 10:25'!
findClassCategory: aCat
	"Return the node if the item matches. By default, search among the kids. Is lazy and will avoid triggering child building."

	item == aCat
		ifTrue: [ ^ self ]
		ifFalse: [ ^ nil ]! !

ABAbstractNode subclass: #ABClassCommentNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassCommentNode commentStamp: '<historical>' prior: 0!
An ABClassCommentNode is for a class comment, if any. No children.
!

!ABClassCommentNode methodsFor: 'actions' stamp: 'ThierryGoubier 9/19/2012 21:49'!
accept: aText notifying: aController
	"This should change the class comment."

	self item comment: aText.
	^ self! !

!ABClassCommentNode methodsFor: 'printing' stamp: 'ThierryGoubier 9/18/2012 13:17'!
asString
	^  'comment'! !

!ABClassCommentNode methodsFor: 'printing' stamp: 'ThierryGoubier 9/18/2012 13:17'!
asText
	^ self item hasComment
		ifTrue: [ 'comment' asText addAttribute: TextEmphasis italic ]
		ifFalse: [ 'comment' ]! !

!ABClassCommentNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/27/2012 14:30'!
findClassComment: aClass
	^ item == aClass
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABClassCommentNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/16/2012 20:36'!
sourceCode
	^ self item comment! !

ABAbstractNode subclass: #ABClassNode
	instanceVariableNames: 'indent'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassNode commentStamp: '<historical>' prior: 0!
An ABClassNode represent a class!

ABClassNode subclass: #ABClassExtensionNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABClassExtensionNode commentStamp: '<historical>' prior: 0!
An ABClassExtensionNode is used when browsing the extension of a package. Restrict itself to the methods of the extension of the parent package.!

!ABClassExtensionNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/17/2012 13:37'!
asString
	^ self item asString! !

!ABClassExtensionNode methodsFor: 'private' stamp: 'ThierryGoubier 7/17/2012 13:39'!
childNodeClassFromItem: anItem
	^ ABMethodNode! !

!ABClassExtensionNode methodsFor: 'private' stamp: 'ThierryGoubier 8/27/2012 00:19'!
childrenItems
	^ ((self parentNode item extensionMethodsForClass: self item) collect: [ :e | e selector ])
		asSortedCollection! !

!ABClassExtensionNode methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 9/20/2012 22:03'!
findAllMethodParents: aMethodSelector forClass: aClass
	"Ask the package to know if this method is an extension. Ok, this may work, but only if the class has already extension methods. Is short-circuited by PackageExtensionNode."
	
	^ (item == aClass and: [ parentNode item includesExtensionSelector: aMethodSelector ofClass:  item])
		ifTrue: [ OrderedCollection with: self parentNode ]
		ifFalse: [ nil ]! !

!ABClassExtensionNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/1/2012 00:28'!
findClass: aClass
	"When we are looking for a class, we fail the test in that class extension."

	^ nil! !

!ABClassExtensionNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/20/2012 22:17'!
findMethodCategory: aCategory in: aClass
	"As an extension, we are a protocol (or multiple ones, in fact). Return the parent node, it's safer when the class loses its last extension selector (it's a bit of a hack, really."

	^ (item == aClass and: [ parentNode item isYourClassExtension: aCategory ])
		ifTrue: [ 
			self childrenItems isEmpty
				ifTrue: [ self parentNode ]
				ifFalse: [ self ] ]
		ifFalse: [ nil ]! !

!ABClassExtensionNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/28/2012 17:28'!
methodCategory
	"Still working in fact."
	
	^ parentNode item methodCategoryPrefix! !

!ABClassExtensionNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/31/2012 21:58'!
sourceClass
	^ self item! !

!ABClassExtensionNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/31/2012 22:08'!
sourceCode
	^ 'message selector and argument names
	"comment stating purpose of message"

	| temporary variable names |
	statements'! !

!ABClassNode methodsFor: 'actions' stamp: 'ThierryGoubier 9/8/2012 11:13'!
accept: aText notifying: aController
	| res |
	res := [ 
	UndefinedObject evaluatorClass new
		evaluate: (ReadStream on: aText)
		in: nil
		to: nil
		notifying: aController
		ifFail: [ ^ nil ]
		logged: true ]
		on: OutOfScopeNotification
		do: [ :ex | ex resume: true ].
	res notNil
		ifTrue: [ 
			| r |
			AltBrowser roots detect: [ :aRoot | (r := aRoot findClass: res) notNil ].
			^ r ].
	^ nil! !

!ABClassNode methodsFor: 'printing' stamp: 'ThierryGoubier 9/10/2012 13:14'!
asString
	^ self item isMeta
		ifTrue: [ '- Class' ]
		ifFalse: [ (String new: self indent withAll: Character space) , self item asString ]! !

!ABClassNode methodsFor: 'private' stamp: 'ThierryGoubier 7/16/2012 20:07'!
childNodeClassFromItem: anItem
	anItem == self item classSide
		ifTrue: [ ^ self class ].
	anItem == self item
		ifTrue: [ ^ ABClassCommentNode ].
	^ ABMethodCategoryNode! !

!ABClassNode methodsFor: 'private' stamp: 'ThierryGoubier 7/16/2012 20:06'!
childrenItems
	^ self item isMeta
		ifFalse: [ 
			{(self item classSide).
			(self item)} , self item organization categories ]
		ifTrue: [ self item organization categories ]! !

!ABClassNode methodsFor: 'commands' stamp: 'ThierryGoubier 9/10/2012 22:09'!
classSearchCommands
	<treeAreaCommand>
	^ {ABClassRefsCommand}! !

!ABClassNode methodsFor: 'private' stamp: 'ThierryGoubier 7/16/2012 13:56'!
classSide
	^ self item isMeta
		ifFalse: [ {(self item classSide)} ]
		ifTrue: [ {} ]! !

!ABClassNode methodsFor: 'commands' stamp: 'ThierryGoubier 9/16/2012 08:06'!
clipboardCommands
	<treeAreaCommand>
	^ {ABRemoveClassCommand}! !

!ABClassNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/3/2012 17:45'!
contentsOrganization
	"Answer the organization of that object. This is the cached one (as probably displayed), maybe not the real one."

	^ Array
		withAll:
			((self item isMeta
				ifTrue: [ self contents ]
				ifFalse: [ self contents allButFirst: 2 ]) collect: [ :c | c item ])! !

!ABClassNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/1/2012 18:38'!
findAllClass: aClass
	^ self item == aClass
		ifTrue: [ OrderedCollection with: self ]
		ifFalse: [ super findAllClass: aClass ]! !

!ABClassNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/1/2012 18:38'!
findClass: aClass
	^ self item == aClass
		ifTrue: [ self ]
		ifFalse: [ super findClass: aClass ]! !

!ABClassNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 10:18'!
findMethod: aSelector in: aClass
	"Ok. Forced version. May need to force on a case by case basis."

	| res |
	self item isMeta
		ifTrue: [ 
			aClass == item
				ifTrue: [ 
					self contents
						do: [ :each | 
							(res := each findMethod: aSelector in: aClass) notNil
								ifTrue: [ ^ res ]] ] ]
		ifFalse: [ 
			aClass == item
				ifTrue: [ 
					self contents
						do: [ :each | 
							(res := each findMethod: aSelector in: aClass) notNil
								ifTrue: [ ^ res ] ] ]
				ifFalse: [ ^ self contents first findMethod: aSelector in: aClass ] ].
	^ nil! !

!ABClassNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/10/2012 13:16'!
indent
	indent isNil
		ifTrue: [ 
			| i |
			(i := self parentNode contents detect: [ :e | e item == self item superclass ] ifNone: [ nil ]) notNil
				ifTrue: [ indent := i indent + 1 ]
				ifFalse: [ indent := 0 ] ].
	^ indent! !

!ABClassNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/19/2012 23:23'!
isStyled
	^ true! !

!ABClassNode methodsFor: 'commands' stamp: 'ThierryGoubier 9/15/2012 11:32'!
navigationCommands
	<treeAreaCommand>
	^ {ABGoToSuperclassCommand.
	ABGoToSubclassCommand.
	ABNextSiblingClassCommand}! !

!ABClassNode methodsFor: 'private' stamp: 'ThierryGoubier 7/17/2012 13:54'!
privateClass
	^ self item! !

!ABClassNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/16/2012 14:18'!
sourceCode
	^ self item definition! !

!ABClassNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/21/2012 16:18'!
title
	^ item name! !

ABAbstractNode subclass: #ABMethodCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABMethodCategoryNode commentStamp: '<historical>' prior: 0!
For a method category. No need for an -- all ---, the tree list allow for opening everything if needed. The 'as yet unclassified' method category (protocol) is managed on the fly (and exist in the system) !

!ABMethodCategoryNode methodsFor: 'actions' stamp: 'ThierryGoubier 9/22/2012 11:58'!
accept: aText notifying: aController
	"Compile the method. Return a target methodNode object for the method. We know it should be one of our children."

	| aSelector aMethodNode |
	aSelector := self sourceClass compile: aText classified: self methodCategory notifying: aController.	"New node should have been added in there."
	self rebuildContents.
	aMethodNode := self parentNode findMethod: aSelector in: self sourceClass.
	^ aMethodNode! !

!ABMethodCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 7/16/2012 00:18'!
childNodeClassFromItem: anItem
	^ ABMethodNode! !

!ABMethodCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 7/27/2012 16:25'!
childrenItems
	^ (self parentNode item methodsInCategory: self item) asSortedCollection! !

!ABMethodCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/3/2012 17:39'!
contentsOrganization
	^ self item -> self childrenItems! !

!ABMethodCategoryNode methodsFor: 'drag and drop' stamp: 'ThierryGoubier 9/22/2012 09:39'!
dropItem: anItem
	"Only valid for an ABMethodCategoryNode or an ABMethodNode."

	((anItem isKindOf: ABMethodCategoryNode) or: [ anItem isKindOf: ABMethodNode ])
		ifFalse: [ ^ self ].
	anItem sourceClass == self sourceClass
		ifFalse: [ ^ self ].
	(anItem isKindOf: ABMethodNode)
		ifTrue: [ 
			| cm |
			cm := anItem compiledMethod.
			cm methodClass organization classify: anItem item under: self item suppressIfDefault: false ]! !

!ABMethodCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 8/29/2012 11:21'!
findAllMethodParents: aMethodSelector forClass: aClass
	^ (parentNode item == aClass and: [ (parentNode item whichCategoryIncludesSelector: aMethodSelector) = item ])
		ifTrue: [ OrderedCollection with: self ]
		ifFalse: [ nil ]! !

!ABMethodCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 17:09'!
findMethod: aSelector in: aClass
	self sourceClass == aClass
		ifTrue: [ 
			self contents
				do: [ :each | 
					| res |
					(res := each findMethod: aSelector in: aClass) notNil
						ifTrue: [ ^ res ] ] ].
	^ nil! !

!ABMethodCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/6/2012 22:36'!
findMethodCategory: aMethodCategory in: aClass
	^ (aMethodCategory = item and: [ aClass == parentNode item ])
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABMethodCategoryNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 22:46'!
isStyled
	^ true! !

!ABMethodCategoryNode methodsFor: 'accessing' stamp: 'ThierryGoubier 7/31/2012 22:00'!
methodCategory
	^ item! !

!ABMethodCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 7/17/2012 13:54'!
privateClass
	^ self parentNode privateClass! !

!ABMethodCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/17/2012 13:53'!
sourceClass
	^ self parentNode item! !

!ABMethodCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/31/2012 22:09'!
sourceCode
	^ 'message selector and argument names
	"comment stating purpose of message"

	| temporary variable names |
	statements'! !

!ABMethodCategoryNode methodsFor: 'commands' stamp: 'ThierryGoubier 8/30/2012 16:15'!
treeAreaCommands
	<treeAreaCommand>
	^ {ABAddProtocolCommand. ABRemoveProtocolCommand }! !

ABAbstractNode subclass: #ABMethodNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABMethodNode commentStamp: '<historical>' prior: 0!
Represent a method. Should be used to get the method source, selector and whatever.!

!ABMethodNode methodsFor: 'actions' stamp: 'ThierryGoubier 9/22/2012 10:06'!
accept: aText notifying: aController
	"Compile the method. All accept: methods should return the relevant object created or modified, for tracking by the GUI."

	"There is something about notifying here."

	| aSelector |
	aSelector := self sourceClass compile: aText classified: self parentNode methodCategory notifying: aController.
	self parentNode rebuildContents.
	^ self parentNode contents detect: [ :each | each item == aSelector ]! !

!ABMethodNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/22/2012 09:32'!
compiledMethod
	^ self parentNode privateClass compiledMethodAt: self item! !

!ABMethodNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/7/2012 14:16'!
findMethod: anItem in: aClass
	^ (item == anItem and: [ self sourceClass == aClass ])
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABMethodNode methodsFor: 'testing' stamp: 'ThierryGoubier 7/19/2012 23:22'!
isStyled
	^ true! !

!ABMethodNode methodsFor: 'commands' stamp: 'ThierryGoubier 9/19/2012 10:33'!
navigationCommands
	<treeAreaCommand>
	^ {ABGoToSuperMethodCommand.
	ABImplementorsCommand.
	ABSendersCommand.
	ABBrowseVersionsCommand}! !

!ABMethodNode methodsFor: 'commands' stamp: 'ThierryGoubier 9/6/2012 22:12'!
removeCommand
	<treeAreaCommand>
	^ {ABRemoveMethodCommand}! !

!ABMethodNode methodsFor: 'commands' stamp: 'ThierryGoubier 9/14/2012 22:46'!
searchTextCommands
	<textAreaCommand>
	^ {ABImplementorsTextCommand}! !

!ABMethodNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/17/2012 13:55'!
sourceClass
	"Get the class through the parent."

	^ (self parentNode privateClass compiledMethodAt: self item) methodClass! !

!ABMethodNode methodsFor: 'querying' stamp: 'ThierryGoubier 7/17/2012 13:55'!
sourceCode
	"Get the source code through the parent."

	^ (self parentNode privateClass compiledMethodAt: self item) sourceCode! !

!ABMethodNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/21/2012 16:13'!
title
	^ self sourceClass name, '>>', item! !

ABAbstractNode subclass: #ABPackageCategoryNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABPackageCategoryNode commentStamp: '<historical>' prior: 0!
An ABPackageCategoryNode represent a category of packages.

item contains an association "package category name" -> "list of packages.".

!

!ABPackageCategoryNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 09:44'!
addChild: anItem
	"Add a child, but do it according to it's position."

	| d |
	item value add: anItem.
	d := self childrenItems indexOf: anItem.
	contents isNil ifTrue: [^nil] ifFalse: [
	^ contents add: (self childNodeFromItem: anItem) beforeIndex: d]! !

!ABPackageCategoryNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 10:21'!
addChildItem: anItem
	"Add a child, but do it according to it's position."

	| d |
	self contents.	"Force contents creation."
	item value add: anItem item.
	anItem parentNode: self.
	^ contents add: anItem! !

!ABPackageCategoryNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/30/2012 06:59'!
addChildItem: anItem beforeIndex: anIndex
	"Add a child, but do it according to it's position."

	| d |
	self contents.	"Force contents creation."
	item value add: anItem item beforeIndex: anIndex.
	anItem parentNode: self.
	^ contents add: anItem beforeIndex: anIndex! !

!ABPackageCategoryNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/14/2012 23:17'!
asString
	^ item key! !

!ABPackageCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 9/19/2012 22:23'!
childNodeClassFromItem: anItem
	(anItem isKindOf: Association )
		ifTrue: [ ^ self class ].
	^ ABPackageSetNode! !

!ABPackageCategoryNode methodsFor: 'private' stamp: 'ThierryGoubier 7/14/2012 23:18'!
childrenItems
	^ item value! !

!ABPackageCategoryNode methodsFor: 'drag and drop' stamp: 'ThierryGoubier 9/9/2012 23:34'!
dropItem: anItem
	"Ok. If we receive a category or a package we either add it to its
	children (if it has no children ?) or add it just before this item?"

	((anItem isKindOf: ABPackageCategoryNode) or: [ anItem isKindOf: ABPackageNode ])
		ifFalse: [ ^ nil ].
	anItem parentNode isNil
		ifTrue: [ AltBrowser roots remove: anItem ]
		ifFalse: [ anItem parentNode removeChildItem: anItem ].
	AltBrowser announcer announce: (AltNodeRemoved node: anItem).
	self childrenItems isEmpty
		ifTrue: [ self addChildItem: anItem ]
		ifFalse: [ 
			self parentNode isNil
				ifTrue: [ 
					AltBrowser roots add: anItem before: self.
					anItem parentNode: nil ]
				ifFalse: [ self parentNode addChildItem: anItem beforeIndex: (self parentNode contents indexOf: self) ] ].
	AltBrowser announcer announce: (AltNodeAdded node: anItem).
	^ anItem! !

!ABPackageCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 16:21'!
findMethod: aSelector in: aClass
	"Return the node if the item matches. By default, search among the kids."

	"Not  lazy : force contents generation."

	| res |
	self contents
		do: [ :each | 
			(res := each findMethod: aSelector in: aClass) notNil
				ifTrue: [ ^ res ] ].
	^ nil! !

!ABPackageCategoryNode methodsFor: 'commands' stamp: 'ThierryGoubier 9/9/2012 23:43'!
rcTreeCommands
	<treeAreaCommand>
	^ {ABRemovePackageCategoryCommand.
	ABCreatePackageCategoryCommand}! !

!ABPackageCategoryNode methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 21:24'!
removeChildItem: anItem
	"Remove a child."

	| d |
	item value remove: anItem item.
	contents isNil
		ifTrue: [ ^ nil ]
		ifFalse: [ ^ contents remove: anItem ]! !

!ABPackageCategoryNode methodsFor: 'querying' stamp: 'ThierryGoubier 8/30/2012 22:16'!
sourceCode
	"A description of the group."
	
	^ 'Package group : ', item key, '
	All packages classified under the term ', item key! !

!ABPackageCategoryNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/21/2012 16:19'!
title
	^ item key! !

ABAbstractNode subclass: #ABPackageNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABPackageNode commentStamp: '<historical>' prior: 0!
An ABPackageNode represent a package and all classes matching it (categories).
!

ABPackageNode subclass: #ABPackageExtensionNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABPackageExtensionNode commentStamp: '<historical>' prior: 0!
An ABPackageExtensionNode is for the extensions part of a package.!

!ABPackageExtensionNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/17/2012 13:31'!
asString
	^ '*extensions*'! !

!ABPackageExtensionNode methodsFor: 'private' stamp: 'ThierryGoubier 7/17/2012 13:38'!
childNodeClassFromItem: anItem
	^ ABClassExtensionNode! !

!ABPackageExtensionNode methodsFor: 'private' stamp: 'ThierryGoubier 8/27/2012 00:19'!
childrenItems
	^ self item extendedClasses! !

!ABPackageExtensionNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/20/2012 22:02'!
findAllMethodParents: aSelector forClass: aClass
	"Returns an OrderedCollection for self"

	^ (self item includesExtensionSelector: aSelector ofClass: aClass)
		ifTrue: [ OrderedCollection with: self ]
		ifFalse: [ nil ]! !

!ABPackageNode methodsFor: 'actions' stamp: 'ThierryGoubier 9/23/2012 22:18'!
accept: aText notifying: aController
	| res |
	res := [ 
	UndefinedObject evaluatorClass new
		evaluate: (ReadStream on: aText)
		in: nil
		to: nil
		notifying: aController
		ifFail: [ ^ nil ]
		logged: true ]
		on: OutOfScopeNotification
		do: [ :ex | ex resume: true ].
	res notNil
		ifTrue: [ 
			| r |
			AltBrowser roots detect: [ :aRoot | (r := aRoot findClass: res) notNil ].
			^ r ].
	^ nil! !

!ABPackageNode methodsFor: 'printing' stamp: 'ThierryGoubier 7/15/2012 07:49'!
asString
	^ self item packageName! !

!ABPackageNode methodsFor: 'private' stamp: 'ThierryGoubier 8/31/2012 07:59'!
childNodeClassFromItem: anItem
	((anItem isKindOf: Class) or: [ (anItem isKindOf: Metaclass) or: [ anItem isTrait ] ])
		ifTrue: [ ^ ABClassNode ].
	(anItem isKindOf: RPackage)
		ifTrue: [ ^ ABPackageExtensionNode ].
	^ ABClassCategoryNode! !

!ABPackageNode methodsFor: 'private' stamp: 'ThierryGoubier 9/22/2012 09:09'!
childrenItems
	"Extract from the organisation the various classes. Use the same implementation as the Class Category, if only one category."
	
	"For the time being, do not track tags in RPackages."

	| o |
	o := OrderedCollection new.
	o add: self item.
	o addAll: (ABClassCategoryNode orderClasses: self item definedClasses asOrderedCollection).
	^ o! !

!ABPackageNode methodsFor: 'drag and drop' stamp: 'ThierryGoubier 9/8/2012 11:15'!
dropItem: anItem
	"Ok. If we receive a category, then we add the category at the end of the parent node. It amounts to removing a node from the root items and adding it here."

	((anItem isKindOf: ABPackageCategoryNode) or: [ anItem isKindOf: ABPackageNode ])
		ifFalse: [ ^ self ].
	anItem parentNode isNil
		ifTrue: [ AltBrowser roots remove: anItem ]
		ifFalse: [ anItem parentNode removeChildItem: anItem ].
	AltBrowser announcer announce: (AltNodeRemoved node: anItem).
	self parentNode addChildItem: anItem beforeIndex: (parentNode contents indexOf: self).
	AltBrowser announcer announce: (AltNodeAdded node: anItem)! !

!ABPackageNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 17:15'!
findClass: aClass
	"Filter out: force contents building if the class belongs to this package."
	
	| res |
	(self item includesClass: aClass)
		ifTrue: [ self contents do: [ :each | 
							(res := each findClass: aClass) notNil
								ifTrue: [ ^ res ]] ].
	^ nil! !

!ABPackageNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 16:18'!
findMethod: aSelector in: aClass
	"Filter out: force contents building if the class belongs to this package."
	
	| res |
	(self item includesClass: aClass)
		ifTrue: [ self contents do: [ :each | 
							(res := each findMethod: aSelector in: aClass) notNil
								ifTrue: [ ^ res ]] ].
	^ nil! !

!ABPackageNode methodsFor: 'querying' stamp: 'ThierryGoubier 8/31/2012 07:48'!
findPackage: aRPackage
	"As a package extension node inherit and is set below, then it is never reached."

	^ item == aRPackage
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABPackageNode methodsFor: 'testing' stamp: 'ThierryGoubier 9/23/2012 22:16'!
isStyled
	^ true! !

!ABPackageNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 22:16'!
sourceCode
	"The default class scheme for a package."

	^ 'Object subclass: #NameOfSubclass
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''' , self item packageName, ''''! !

!ABPackageNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/21/2012 16:21'!
title
	^ item packageName! !

ABAbstractNode subclass: #ABPackageSetNode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser'!

!ABPackageSetNode commentStamp: '<historical>' prior: 0!
This class instances represent PackageSets or MCWorkingCopy of a package.!

!ABPackageSetNode methodsFor: 'actions' stamp: 'ThierryGoubier 9/23/2012 22:24'!
accept: aText notifying: aController
	| res |
	res := [ 
	UndefinedObject evaluatorClass new
		evaluate: (ReadStream on: aText)
		in: nil
		to: nil
		notifying: aController
		ifFail: [ ^ nil ]
		logged: true ]
		on: OutOfScopeNotification
		do: [ :ex | ex resume: true ].
	res notNil
		ifTrue: [ 
			| r |
			AltBrowser roots detect: [ :aRoot | (r := aRoot findClass: res) notNil ].
			^ r ].
	^ nil! !

!ABPackageSetNode methodsFor: 'printing' stamp: 'ThierryGoubier 9/21/2012 14:58'!
asString
	(item isKindOf: MCWorkingCopy)
		ifTrue: [^ self item packageNameWithStar ].
	^ item printString! !

!ABPackageSetNode methodsFor: 'private' stamp: 'ThierryGoubier 9/19/2012 22:04'!
childNodeClassFromItem: anItem
	((anItem isKindOf: RPackage) and: [ self item packageSet packages size = 1 ])
		ifTrue: [ ^ ABPackageExtensionNode ].
	(anItem isKindOf: RPackage)
		ifTrue: [ ^ ABPackageNode ].
	^ ABClassNode! !

!ABPackageSetNode methodsFor: 'private' stamp: 'ThierryGoubier 9/19/2012 22:19'!
childrenItems
	| o |
	o := OrderedCollection new.
	self item packageSet packages size = 1
		ifTrue: [ 
			| package |
			package := self item packageSet packages first.
			o add: package.
			o addAll: (ABClassCategoryNode orderClasses: package definedClasses asOrderedCollection) ]
		ifFalse: [ o addAll: self item packageSet packages ].
	^ o! !

!ABPackageSetNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 16:20'!
findMethod: aSelector in: aClass
	"Return the node if the item matches. By default, search among the kids."

	"Not lazy : trigger contents generation."

	| res |
	self contents
		do: [ :each | 
			(res := each findMethod: aSelector in: aClass) notNil
				ifTrue: [ ^ res ] ].
	^ nil! !

!ABPackageSetNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/23/2012 22:59'!
findPackage: aRPackage
	"If a package set has a single package, then the package set takes the package place in the tree and reacts as such."

	| pkSet pks |
	pkSet := item packageSet.
	pks := pkSet packages.
	(pks size = 1 and: [ pks first == aRPackage ])
		ifTrue: [ ^ self ]
		ifFalse: [ ^ super findPackage: aRPackage ]! !

!ABPackageSetNode methodsFor: 'querying' stamp: 'ThierryGoubier 9/21/2012 10:25'!
findPackageSet: aPackageSetOrMCWorkingCopy
	^ (item = aPackageSetOrMCWorkingCopy or: [ item packageSet = aPackageSetOrMCWorkingCopy or: [item package = aPackageSetOrMCWorkingCopy]])
		ifTrue: [ self ]
		ifFalse: [ nil ]! !

!ABPackageSetNode methodsFor: 'testing' stamp: 'ThierryGoubier 9/23/2012 22:25'!
isStyled
	^ true! !

!ABPackageSetNode methodsFor: 'private' stamp: 'ThierryGoubier 9/23/2012 22:26'!
sourceCode
	"The default class scheme for a package."

	^ 'Object subclass: #NameOfSubclass
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''' , self item packageName, ''''! !

!ABPackageSetNode methodsFor: 'accessing' stamp: 'ThierryGoubier 9/21/2012 16:20'!
title
	^ item packageNameWithStar! !

Object subclass: #ABCommand
	instanceVariableNames: 'target requestor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCommand commentStamp: '<historical>' prior: 0!
An ABCommand is in charge of preparing and executing commands on the Alt Browser.

Instance Variables
	requestor:	<Object>	The object sending the command. In my case, the AltBrowser instance.
	target:		<Object>	The target (where the command is applied): the textModel usually (or the treeModel for tree related commands ?)!

ABCommand subclass: #ABAcceptTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABAcceptTextCommand commentStamp: '<historical>' prior: 0!
An ABAcceptTextCommand is about accepting the text.
!

!ABAcceptTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABAcceptTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABAcceptTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/9/2012 23:28'!
execute
	"The execute may create an object or return us one."

	| result selection |
	target widget accept.
	selection := requestor selectedItem item.
	result := selection accept: target getText notifying: target widget.
	(result isNil or: [ result == selection ])
		ifTrue: [ ^ self ].
	requestor selectPathForAbstractItem: result! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:39'!
group
	^ #zip! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallOkIcon! !

!ABAcceptTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:21'!
isActive
	^ true! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:50'!
keystroke
	^ $s ctrl ! !

!ABAcceptTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:09'!
label
	^ 'Accept'! !

ABCommand subclass: #ABAddProtocolCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABAddProtocolCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/20/2012 22:19'!
execute
	"Reuse code from Browser>>addCategory"

	"Present a choice of categories or prompt for a new category name and add it before the current selection, or at the end if no current selection"

	"Usability comment : far too many choices in most cases to be user-friendly."

	| labels reject lines oldIndex newName tClass sItem |
	requestor okToChange
		ifFalse: [ ^ self ].
	labels := OrderedCollection new.
	sItem := requestor selectedItem.
	tClass := requestor selectedItem sourceClass.
	reject := Set new.
	reject
		addAll: tClass organization categories;
		add: ClassOrganizer nullCategory;
		add: ClassOrganizer default.
	lines := OrderedCollection new.
	tClass allSuperclasses
		do: [ :cls | 
			| cats |
			cls = Object
				ifFalse: [ 
					cats := cls organization categories reject: [ :cat | reject includes: cat ].
					(cats isEmpty or: [ reject size > 10 ])
						ifFalse: [ 
							lines add: labels size.
							labels addAll: cats asSortedCollection.
							reject addAll: cats ] ] ].
	newName := UIManager default chooseOrRequestFrom: labels lines: lines title: 'Add Category'.
	newName ifNil: [ ^ self ].
	newName := newName asSymbol.
	tClass
		addCategory: newName
		before:
			((sItem item isKindOf: ABMethodCategoryNode)
				ifTrue: [ sItem item item ]
				ifFalse: [ nil ]).
	requestor selectPathForItem: (sItem parentNode findItem: (sItem parentNode item findMethodCategory: newName in: tClass))! !

!ABAddProtocolCommand methodsFor: 'testing' stamp: 'ThierryGoubier 8/29/2012 10:39'!
isActive
	^ true! !

!ABAddProtocolCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/29/2012 10:39'!
label
	^ 'Add protocol'! !

ABCommand subclass: #ABBrowseVersionsCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABBrowseVersionsCommand commentStamp: '<historical>' prior: 0!
This command opens a version browser on a method.!

!ABBrowseVersionsCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/19/2012 10:34'!
execute
	| class selector |
	class := requestor selectedItem sourceClass.
	selector := requestor selectedItem item item.
	Smalltalk tools versionBrowser
		browseVersionsOf: (class compiledMethodAt: selector)
		class: class theNonMetaClass
		meta: class isMeta
		category: requestor selectedItem item parentNode methodCategory
		selector: selector! !

!ABBrowseVersionsCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/19/2012 10:24'!
icon
	^ #smallSystemBrowserIcon! !

!ABBrowseVersionsCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/19/2012 10:34'!
isActive
	^ true! !

!ABBrowseVersionsCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/19/2012 10:33'!
isEnabled
	^ true! !

!ABBrowseVersionsCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/19/2012 10:24'!
keystroke
	^ $v command! !

!ABBrowseVersionsCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/19/2012 10:24'!
label
	^ 'Versions'! !

ABCommand subclass: #ABCancelTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCancelTextCommand commentStamp: '<historical>' prior: 0!
An ABCancelTextCommand is about cancelling the last action.!

!ABCancelTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABCancelTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABCancelTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/2/2012 13:34'!
execute
	target widget cancel! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:39'!
group
	^ #zip! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallCancelIcon! !

!ABCancelTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:51'!
keystroke
	^ $l ctrl! !

!ABCancelTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:11'!
label
	^ 'Cancel'! !

ABCommand subclass: #ABClassRefsCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABClassRefsCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/10/2012 16:12'!
execute
	"Strange, but convenient, to see that systemNavigation is defined for every object."

	| class |
	class := requestor selectedItem item item theNonMetaClass ifNil: [ ^ self ].
	class isTrait
		ifTrue: [ self systemNavigation browseAllUsersOfTrait: class ]
		ifFalse: [ self systemNavigation browseAllCallsOnClass: class ]! !

!ABClassRefsCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/10/2012 16:09'!
icon
	^ #smallFindIcon! !

!ABClassRefsCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/10/2012 16:26'!
isActive
	^ true! !

!ABClassRefsCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/10/2012 16:15'!
isEnabled
	^ true! !

!ABClassRefsCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/10/2012 16:02'!
label
	^ 'Class refs'! !

!ABClassRefsCommand methodsFor: 'user interface' stamp: 'ThierryGoubier 9/10/2012 16:50'!
neo! !

!ABCommand class methodsFor: 'instance creation' stamp: 'cwp 10/14/2006 23:29'!
on: target for: requestor
	^ self new setTarget: target requestor: requestor! !

!ABCommand methodsFor: 'user interface' stamp: 'ThierryGoubier 9/11/2012 13:10'!
addItemToMenu: aMenu
	(self isActive and: [ self wantsMenu ])
		ifTrue: [ 
			(aMenu add: self label target: self selector: #execute)
				enablementSelector: #isEnabled;
				icon:
						(self icon notNil
								ifTrue: [ UITheme current perform: self icon ]
								ifFalse: [ nil ]);
				keyText: self keyString ]! !

!ABCommand methodsFor: 'user interface' stamp: 'ThierryGoubier 9/12/2012 16:25'!
buildShortcut: aKMDispatcher
	"Add a shortcut to the keymap. Conditions : must have a keystroke, must wantsKeyboard and must be active."

	(self keystroke isNil or: [ self wantsKeyboard not or: [ self isActive not ] ])
		ifTrue: [ ^ self ].
	aKMDispatcher
		on: self keystroke asShortcut
		do: [ self execute ]! !

!ABCommand methodsFor: 'execution' stamp: 'cwp 10/15/2006 14:12'!
execute
	! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 10/6/2006 18:45'!
icon
	^ nil! !

!ABCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 22:05'!
isActive
	^ false! !

!ABCommand methodsFor: 'testing' stamp: 'cwp 10/15/2006 19:31'!
isEnabled
	^ true! !

!ABCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/7/2012 22:49'!
keyString
	| c |
	^ (c := self keystroke) isNil
		ifFalse: [ c asString copyWithout: Character space ]
		ifTrue: [ nil ]! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 9/27/2006 22:53'!
keystroke
	^ nil
! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 9/27/2006 22:53'!
label
	self subclassResponsibility
! !

!ABCommand methodsFor: 'accessing' stamp: 'cwp 10/14/2006 23:39'!
on: anObject for: aRequestor
	^ self class on: anObject for: aRequestor! !

!ABCommand methodsFor: 'user interface' stamp: 'cwp 10/15/2006 12:37'!
perform: aSelector orSendTo: anObject
	self perform: aSelector! !

!ABCommand methodsFor: 'user interface' stamp: 'lr 11/25/2010 23:45'!
select: aNode with: anAnnouncer
	aNode announceSelectionWith: anAnnouncer! !

!ABCommand methodsFor: 'initialize-release' stamp: 'lr 7/3/2009 22:27'!
setTarget: anObject requestor: aRequestor 
	target := anObject.
	requestor := aRequestor! !

!ABCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/11/2012 13:11'!
wantsButton
	"Select for button use. Reserved for future use (a menu bar ?)."

	^ false
! !

!ABCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/11/2012 13:11'!
wantsKeyboard
	"Select for keyboard use."

	^ true
! !

!ABCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/11/2012 13:07'!
wantsMenu
	"true if the command wants to be in Menus."

	^ true
! !

ABCommand subclass: #ABCopyTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCopyTextCommand commentStamp: '<historical>' prior: 0!
This class does a copy on the text area.!

!ABCopyTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABCopyTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABCopyTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/2/2012 13:35'!
execute
	target widget copySelection! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallCopyIcon! !

!ABCopyTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABCopyTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 07:46'!
isEnabled
	^ target widget trueSelection isEmpty not! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:51'!
keystroke
	^ $c ctrl! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:11'!
label
	^ 'Copy'! !

!ABCopyTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:29'!
order
	^ '2'! !

ABCommand subclass: #ABCreatePackageCategoryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCreatePackageCategoryCommand commentStamp: '<historical>' prior: 0!
This class prompts for a name and creates a category at top level.
!

!ABCreatePackageCategoryCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/8/2012 11:14'!
execute
	"Ask for a category name and create it."

	| name |
	name := UIManager default request: 'New Category name?'.
	name notNil
		ifTrue: [ 
			| item |
			item := ABPackageCategoryNode with: name -> OrderedCollection new.
			requestor selectedItem notNil
				ifTrue: [ AltBrowser roots add: item afterIndex: (AltBrowser roots indexOf: requestor selectedItem item) ]
				ifFalse: [ AltBrowser roots add: item ].
			AltBrowser announcer announce: (AltNodeAdded node: item) ]! !

!ABCreatePackageCategoryCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/31/2012 15:52'!
icon
	^ nil! !

!ABCreatePackageCategoryCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 15:54'!
isActive
	^ true! !

!ABCreatePackageCategoryCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 15:59'!
isEnabled
	^ requestor selectedItem isNil or: [ requestor selectedItem item isKindOf: ABPackageCategoryNode ]! !

!ABCreatePackageCategoryCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/31/2012 15:58'!
label
	^'Create '! !

ABCommand subclass: #ABCutCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCutCommand commentStamp: '<historical>' prior: 0!
This command starts a move, different from a delete. It's focusing on giving the same behavior as the cut / paste of file browsers.!

!ABCutCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/9/2012 23:35'!
execute
	"Use the AltBrowser temporary to store the item."

	AltBrowser temporary: requestor selectedItem item! !

!ABCutCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/9/2012 22:10'!
icon
	^ #smallCutIcon! !

!ABCutCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/9/2012 22:13'!
isActive
	^ true! !

!ABCutCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/9/2012 22:12'!
isEnabled
	^ true! !

!ABCutCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/9/2012 22:14'!
keystroke
	^ $x ctrl! !

!ABCutCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/9/2012 22:09'!
label
	^ 'Cut'
	! !

ABCommand subclass: #ABCutTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABCutTextCommand commentStamp: '<historical>' prior: 0!
This class does a cut in the text area.
!

!ABCutTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABCutTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABCutTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/2/2012 13:34'!
execute
	target widget cut! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallCutIcon! !

!ABCutTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/19/2012 10:43'!
isActive
	^ true! !

!ABCutTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 07:49'!
isEnabled
	^ target widget trueSelection isEmpty not! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/21/2012 09:58'!
keystroke
	^ $x ctrl! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:12'!
label
	^ 'Cut'! !

!ABCutTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:29'!
order
	^ '1'! !

ABCommand subclass: #ABDoItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABDoItTextCommand commentStamp: '<historical>' prior: 0!
ABDoItTextCommand : execute the code selected
!

ABDoItTextCommand subclass: #ABDebugItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABDebugItTextCommand commentStamp: '<historical>' prior: 0!
This class does a debugIt on the selection in the text area.
!

!ABDebugItTextCommand class methodsFor: 'testing' stamp: 'lr 8/27/2011 10:02'!
takesNodes
	^ false! !

!ABDebugItTextCommand class methodsFor: 'testing' stamp: 'lr 8/27/2011 10:02'!
takesText
	^ true! !

!ABDebugItTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/10/2012 23:12'!
execute
	target widget debugIt! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:17'!
icon
	^ #smallDebugIcon! !

!ABDebugItTextCommand methodsFor: 'testing' stamp: 'lr 8/27/2011 09:58'!
isActive
	^ true! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/10/2012 22:12'!
keystroke
	^ $d command shift! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'lr 8/27/2011 09:58'!
label
	^ 'Debug it'! !

!ABDebugItTextCommand methodsFor: 'accessing' stamp: 'lr 8/27/2011 10:00'!
order
	^ '5'! !

!ABDoItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesNodes
	^ false! !

!ABDoItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesText
	^ true! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/17/2012 11:29'!
doItContext

	^ nil! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/17/2012 11:28'!
doItReceiver

	^ requestor selectedItem sourceClass
		ifNil: [ nil ]
		ifNotNil: [:class | class theNonMetaClass ]! !

!ABDoItTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/17/2012 11:31'!
execute
	"Treat the current selection as an expression; evaluate it."

	| result rcvr ctxt |
	rcvr := self doItReceiver.
	ctxt := self doItContext.
	result := [ 
	rcvr class evaluatorClass new
		evaluate: (ReadStream on: target widget trueSelection)
		in: ctxt
		to: rcvr
		notifying: target widget
		ifFail: [ 
			target widget flash.
			^ nil ]
		logged: true	"don't set it to self.. The receiver can be destroyed as a result of evaluateSelection" ]
		on: OutOfScopeNotification
		do: [ :ex | ex resume: true ].
	^ result! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallDoItIcon! !

!ABDoItTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/25/2012 23:25'!
isActive
	^ target readSelection notNil! !

!ABDoItTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 10:40'!
isEnabled
	^ target widget trueSelection notEmpty! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:51'!
keystroke
	^ $d command! !

!ABDoItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:55'!
label
	^ 'Do it'! !

ABDoItTextCommand subclass: #ABExploreItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABExploreItTextCommand commentStamp: '<historical>' prior: 0!
This class does an exploreIt (evaluate and explore) on the contents selected in the text area.
!

!ABExploreItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 15:08'!
takesNodes
	^ false! !

!ABExploreItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 15:08'!
takesText
	^ true! !

!ABExploreItTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 13:47'!
execute
	super execute explore! !

!ABExploreItTextCommand methodsFor: 'testing' stamp: 'lr 3/6/2011 15:05'!
isActive
	^ true! !

!ABExploreItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:52'!
keystroke
	^ $I command! !

!ABExploreItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 15:05'!
label
	^ 'Explore it'! !

!ABExploreItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 15:05'!
order
	^ '4'! !

ABDoItTextCommand subclass: #ABInspectItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABInspectItTextCommand commentStamp: '<historical>' prior: 0!
This class does an inspectIt (evaluate and inspect the result) on the selection in the text area.
!

!ABInspectItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesNodes
	^ false! !

!ABInspectItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesText
	^ true! !

!ABInspectItTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 13:48'!
execute
	super execute inspect! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallInspectItIcon! !

!ABInspectItTextCommand methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
isActive
	^ true! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:52'!
keystroke
	^ $i command! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
label
	^ 'Inspect it'! !

!ABInspectItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
order
	^ '3'! !

ABDoItTextCommand subclass: #ABPrintItTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABPrintItTextCommand commentStamp: '<historical>' prior: 0!
This command does a printIt (evaluate and print the result) on the selection in the text area.!

!ABPrintItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesNodes
	^ false! !

!ABPrintItTextCommand class methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
takesText
	^ true! !

!ABPrintItTextCommand methodsFor: 'execution' stamp: 'lr 3/6/2011 15:13'!
execute
	target printIt! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallPrintIcon! !

!ABPrintItTextCommand methodsFor: 'testing' stamp: 'lr 3/6/2011 14:55'!
isActive
	^ true! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:53'!
keystroke
	^ $p command! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
label
	^ 'Print it'! !

!ABPrintItTextCommand methodsFor: 'accessing' stamp: 'lr 3/6/2011 14:56'!
order
	^ '2'! !

ABCommand subclass: #ABExploreItemCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABExploreItemCommand commentStamp: '<historical>' prior: 0!
An ABExploreItemCommand does an explore of the selected item in the tree pane.
!

!ABExploreItemCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/30/2012 16:47'!
execute
	requestor selectedItem item explore! !

!ABExploreItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/30/2012 16:51'!
icon
	^ #smallInspectItIcon! !

!ABExploreItemCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/19/2012 10:43'!
isActive
	^ AltBrowser debug! !

!ABExploreItemCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/30/2012 16:48'!
isEnabled
	^ requestor selectedItem notNil! !

!ABExploreItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 22:07'!
keystroke
	^ $l command! !

!ABExploreItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/30/2012 16:49'!
label
	^ 'ExploreIt'! !

ABCommand subclass: #ABFindClassCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABFindClassCommand commentStamp: '<historical>' prior: 0!
This command does a find class. Use an openPath approach.!

!ABFindClassCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/25/2012 13:44'!
execute
	"Find a class, then determine the path to reach it, and open this path. Select only one node, and if there is more than one match, use the last match (multiple matches means in current RPackage an additional, incorrect, overall package which is usually before the real, most specific package)."

	"In fact, this is costly (scan of the complete tree) even if it is fast enough, since sometimes the over package is before (Alt), sometimes it is after (Rpackage). It may be irrelevant with evolutions in RPackage."

	| aClass found aNode nPath tmPath |
	aClass := SearchFacade classSearch chooseFromOwner: requestor window window.
	aClass isNil
		ifTrue: [ ^ self ].
	AltBrowser roots
		do: [ :each | 
			| res |
			(res := each findClass: aClass) notNil
				ifTrue: [ found := res ] ].
	found isNil
		ifTrue: [ ^ aClass explore ].
	requestor selectPathForAbstractItem: found! !

!ABFindClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/7/2012 22:51'!
icon
	^ nil! !

!ABFindClassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 8/7/2012 22:50'!
isActive
	^ true! !

!ABFindClassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 8/7/2012 22:50'!
isEnabled
	^ true! !

!ABFindClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/15/2012 10:27'!
keystroke
	^ $f command, $c asShortcut! !

!ABFindClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/7/2012 22:50'!
label
	^ 'Find Class'! !

ABCommand subclass: #ABGoToSubclassCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABGoToSubclassCommand commentStamp: '<historical>' prior: 0!
This command brings you to the first subclass of a class
!

!ABGoToSubclassCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/8/2012 11:14'!
execute
	"To be done."
	
	| found aClass nPath aNode tmPath |
	aClass := requestor selectedItem item item subclasses first.
	AltBrowser roots
		do: [ :each | 
			| res |
			(res := each findClass: aClass) notNil
				ifTrue: [ found := res ] ].
	nPath := OrderedCollection new.
	aNode := found.
	[ aNode notNil ]
		whileTrue: [ 
			nPath add: aNode.
			aNode := aNode parentNode ].
	tmPath := OrderedCollection with: (target getRoots detect: [ :r | r item == nPath last ]).
	nPath removeLast.
	nPath reverseDo: [ :n | tmPath add: (tmPath last forceFindItem: n) ].
	target widget selectPath: tmPath in: (target widget scroller submorphs at: 1 ifAbsent: [ ^ self ])! !

!ABGoToSubclassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/5/2012 14:49'!
icon
	^ nil! !

!ABGoToSubclassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/5/2012 16:45'!
isActive
	^ true! !

!ABGoToSubclassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/5/2012 16:46'!
isEnabled
	^ requestor selectedItem item item subclasses notEmpty! !

!ABGoToSubclassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/15/2012 11:57'!
keystroke
	^ $n command, $u! !

!ABGoToSubclassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/5/2012 14:50'!
label
	^ 'Subclass'! !

ABCommand subclass: #ABGoToSuperMethodCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABGoToSuperMethodCommand commentStamp: '<historical>' prior: 0!
This class does a navigation move to a superclass method of the same name (overridden then by the current one)!

!ABGoToSuperMethodCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/8/2012 11:15'!
execute
	"Move to that place. Can only be called if enabled."

	| tClass tClassNode tMethodNode |
	tClass := self findTargetClass.
	self assert: tClass notNil.
	AltBrowser roots detect: [ :each | (tClassNode := each findClass: tClass) notNil ].
	tMethodNode := tClassNode findMethod: requestor selectedItem item item in: tClass.
	requestor selectPathForAbstractItem: tMethodNode! !

!ABGoToSuperMethodCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/7/2012 14:08'!
findTargetClass
	"Move to that place. We will check if this works for selection first."
	
	requestor selectedItem sourceClass
		allSuperclassesDo: [ :each | 
			(each includesSelector: requestor selectedItem item item)
				ifTrue: [ ^ each ] ].
	^ nil! !

!ABGoToSuperMethodCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/6/2012 21:46'!
icon
	^ nil! !

!ABGoToSuperMethodCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/6/2012 23:08'!
isActive
	^ true! !

!ABGoToSuperMethodCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/6/2012 23:07'!
isEnabled
	requestor selectedItem sourceClass
		allSuperclassesDo: [ :each | 
			(each includesSelector: requestor selectedItem item item)
				ifTrue: [ ^ true ] ].
	^ false! !

!ABGoToSuperMethodCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/15/2012 11:57'!
keystroke
	^ $n command, $s! !

!ABGoToSuperMethodCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/6/2012 21:46'!
label
	^ 'Super ...'! !

ABCommand subclass: #ABGoToSuperclassCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABGoToSuperclassCommand commentStamp: '<historical>' prior: 0!
This command moves the selection to the superclass.
!

!ABGoToSuperclassCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/8/2012 11:14'!
execute
	"Find the superclass and jump to it."

	| found aClass nPath aNode tmPath |
	aClass := requestor selectedItem item item superclass.
	AltBrowser roots contents
		do: [ :each | 
			| res |
			(res := each findClass: aClass) notNil
				ifTrue: [ found := res ] ].
	nPath := OrderedCollection new.
	aNode := found.
	[ aNode notNil ]
		whileTrue: [ 
			nPath add: aNode.
			aNode := aNode parentNode ].
	tmPath := OrderedCollection with: (target getRoots detect: [ :r | r item == nPath last ]).
	nPath removeLast.
	nPath reverseDo: [ :n | tmPath add: (tmPath last forceFindItem: n) ].
	target widget selectPath: tmPath in: (target widget scroller submorphs at: 1 ifAbsent: [ ^ self ])! !

!ABGoToSuperclassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/4/2012 12:19'!
icon
	^ nil! !

!ABGoToSuperclassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/4/2012 13:03'!
isActive
	^ true! !

!ABGoToSuperclassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/4/2012 13:03'!
isEnabled
	^ true! !

!ABGoToSuperclassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/15/2012 11:58'!
keystroke
	^ $n command, $s! !

!ABGoToSuperclassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/4/2012 12:58'!
label
	^ 'Superclass'! !

ABCommand subclass: #ABImplementorsCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABImplementorsCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/7/2012 16:32'!
execute
	SystemNavigation default browseAllImplementorsOf: requestor selectedItem item item! !

!ABImplementorsCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/10/2012 22:10'!
icon
	^ #smallFindIcon! !

!ABImplementorsCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/7/2012 16:32'!
isActive
	^ true! !

!ABImplementorsCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/7/2012 16:32'!
isEnabled
	^ true! !

!ABImplementorsCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/7/2012 16:31'!
label
	^ 'Implementors'! !

ABCommand subclass: #ABImplementorsTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABImplementorsTextCommand commentStamp: '<historical>' prior: 0!
Browse all implementors of the selected selector.!

!ABImplementorsTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/14/2012 22:42'!
execute
	target widget implementorsOfIt! !

!ABImplementorsTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/14/2012 22:44'!
isActive
	^ true! !

!ABImplementorsTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/14/2012 22:43'!
isEnabled
	^ true! !

!ABImplementorsTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/14/2012 22:41'!
keystroke
	^ $m ctrl! !

!ABImplementorsTextCommand methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 9/14/2012 22:39'!
label
	^ 'Implementors...'! !

ABCommand subclass: #ABNewClassCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABNewClassCommand commentStamp: '<historical>' prior: 0!
This command creates a new class
!

ABCommand subclass: #ABNextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABNextCommand commentStamp: '<historical>' prior: 0!
History next command.
!

!ABNextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/4/2012 15:38'!
execute
	requestor selectedItem: requestor history next! !

!ABNextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/4/2012 15:34'!
icon
	^ #smallForwardIcon! !

!ABNextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/4/2012 15:37'!
isActive
	^ true! !

!ABNextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/4/2012 15:37'!
isEnabled
	^ requestor history hasNext! !

!ABNextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/15/2012 11:55'!
keystroke
	^ Character arrowRight command! !

!ABNextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/4/2012 15:37'!
label
	^ 'Forward'! !

ABCommand subclass: #ABNextSiblingClassCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABNextSiblingClassCommand commentStamp: '<historical>' prior: 0!
This command jumps to the sibling class according to the inheritance: the next class which has the same superclass than the selected class.!

!ABNextSiblingClassCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/15/2012 11:34'!
execute
	| found aClass aClassIndex siblings nextClass |
	aClass := requestor selectedItem item item.
	siblings := aClass superclass subclasses.
	aClassIndex := siblings indexOf: aClass.
	nextClass := siblings at: ((aClassIndex rem: siblings size) + 1).
	AltBrowser roots
		do: [ :each | 
			| res |
			(res := each findClass: nextClass) notNil
				ifTrue: [ found := res ] ].
	requestor selectPathForAbstractItem: found! !

!ABNextSiblingClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/15/2012 11:22'!
icon
	^ nil! !

!ABNextSiblingClassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/15/2012 11:32'!
isActive
	^ true! !

!ABNextSiblingClassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/15/2012 11:35'!
isEnabled
	^ requestor selectedItem item item superclass notNil! !

!ABNextSiblingClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/15/2012 11:22'!
keystroke
	^ $n command, $d! !

!ABNextSiblingClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/15/2012 11:21'!
label
	^ 'Next sibling class'! !

ABCommand subclass: #ABPasteCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABPasteCommand commentStamp: '<historical>' prior: 0!
This command paste something which has been cut first.!

!ABPasteCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/9/2012 23:32'!
execute
	"Get the temporary from the AltBrowser and try a move (a drop, maybe ?)"

	(requestor selectedItem item dropItem: AltBrowser temporary) isNil
		ifFalse: [ AltBrowser temporary: nil ]! !

!ABPasteCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/9/2012 23:23'!
icon
	^ #smallPasteIcon! !

!ABPasteCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/9/2012 23:24'!
isActive
	^ true! !

!ABPasteCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/9/2012 23:25'!
isEnabled
	^ true! !

!ABPasteCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/9/2012 23:26'!
keystroke
	^ $v ctrl! !

!ABPasteCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/9/2012 23:22'!
label
	^ 'Paste'
	! !

ABCommand subclass: #ABPasteTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABPasteTextCommand commentStamp: '<historical>' prior: 0!
An ABPasteTextCommand : Paste text from Clipboard. Directly target the editor; neither the model nor the widget have methods for handling that.
!

!ABPasteTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABPasteTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABPasteTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 08:03'!
execute
	target widget textMorph editor replace: target widget trueSelectionInterval with: Clipboard clipboardText and: [  ]! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:27'!
group
	^ #clipboard! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallPasteIcon! !

!ABPasteTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABPasteTextCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/26/2012 07:52'!
isEnabled
	^ Clipboard clipboardText notEmpty! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:53'!
keystroke
	^ $v ctrl! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:14'!
label
	^ 'Paste'! !

!ABPasteTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:29'!
order
	^ '3'! !

ABCommand subclass: #ABPreviousCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABPreviousCommand commentStamp: '<historical>' prior: 0!
History previous in the Alt Browser
!

!ABPreviousCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/11/2012 10:08'!
execute
	requestor selectPathForItem: requestor history previous key! !

!ABPreviousCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/4/2012 15:27'!
icon
	^ #smallBackIcon! !

!ABPreviousCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/4/2012 15:32'!
isActive
	^ true! !

!ABPreviousCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/4/2012 15:33'!
isEnabled
	^ requestor history hasPrevious! !

!ABPreviousCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/15/2012 11:54'!
keystroke 
	^ Character arrowLeft command! !

!ABPreviousCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/4/2012 15:22'!
label
	^ 'Back'! !

ABCommand subclass: #ABRebuildItemCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABRebuildItemCommand commentStamp: '<historical>' prior: 0!
This class forces a rebuild of an item in the tree.
!

!ABRebuildItemCommand methodsFor: 'execution' stamp: 'ThierryGoubier 8/30/2012 16:05'!
execute
	"requestor selectedItem rebuildChildren."
	AltBrowser aNodeToRebuilt: requestor selectedItem item! !

!ABRebuildItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/5/2012 09:08'!
icon
	^ #smallDebugIcon! !

!ABRebuildItemCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/19/2012 10:43'!
isActive
	^ AltBrowser debug! !

!ABRebuildItemCommand methodsFor: 'testing' stamp: 'ThierryGoubier 8/5/2012 09:01'!
isEnabled
	^ requestor selectedItem notNil! !

!ABRebuildItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/5/2012 08:59'!
keystroke
	^ nil! !

!ABRebuildItemCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/5/2012 09:00'!
label
	^ 'Rebuild It'! !

ABCommand subclass: #ABRedoTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABRedoTextCommand commentStamp: '<historical>' prior: 0!
An ABRedoTextCommand calls the text editor redo command.
!

!ABRedoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABRedoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABRedoTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 08:07'!
execute
	target widget textMorph editor redo! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:38'!
group
	^ #adapting! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:18'!
icon
	^ #smallRedoIcon! !

!ABRedoTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:54'!
keystroke
	^ $j ctrl! !

!ABRedoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:20'!
label
	^ 'Redo'! !

ABCommand subclass: #ABRemoveClassCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABRemoveClassCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/16/2012 08:04'!
execute
	| message className classToRemove result |
	requestor okToChange
		ifFalse: [ ^ false ].
	classToRemove := requestor selectedItem item item
		ifNil: [ 
			Beeper beep.
			^ nil ].
	classToRemove := classToRemove theNonMetaClass.
	className := classToRemove name.
	message := 'Are you certain that you
want to REMOVE the class ' , className
		,
			'
from the system?'.
	(result := self confirm: message)
		ifTrue: [ 
			classToRemove subclasses size > 0
				ifTrue: [ 
					(self confirm: 'class has subclasses: ' , message)
						ifFalse: [ ^ nil ] ].
			classToRemove removeFromSystem.
			true ].
	^ nil! !

!ABRemoveClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/20/2012 21:50'!
icon
	^ #smallDeleteIcon! !

!ABRemoveClassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/16/2012 08:05'!
isActive
	^ true! !

!ABRemoveClassCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/16/2012 08:05'!
isEnabled
	^ true! !

!ABRemoveClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/17/2012 22:00'!
keystroke
	^ Character delete asShortcut! !

!ABRemoveClassCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/16/2012 08:01'!
label
	^ 'Remove'! !

ABCommand subclass: #ABRemoveMethodCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABRemoveMethodCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/6/2012 22:14'!
execute
	"As simple as that ?"
	
	requestor selectedItem sourceClass removeSelector: requestor selectedItem item item! !

!ABRemoveMethodCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/20/2012 21:49'!
icon
	^ #smallDeleteIcon! !

!ABRemoveMethodCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/6/2012 22:13'!
isActive
	^ true! !

!ABRemoveMethodCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/6/2012 22:10'!
isEnabled
	^ true! !

!ABRemoveMethodCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/17/2012 22:00'!
keystroke
	^ Character delete asShortcut! !

!ABRemoveMethodCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/6/2012 22:06'!
label
	^ 'Remove'! !

ABCommand subclass: #ABRemovePackageCategoryCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABRemovePackageCategoryCommand commentStamp: '<historical>' prior: 0!
This class does a remove item on the selected item (package category : anything else requires specific system commands.)!

!ABRemovePackageCategoryCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/8/2012 11:16'!
execute
	"Remove the package category."

	| item |
	item := requestor selectedItem item.
	item parentNode notNil
		ifTrue: [ item parentNode removeChildItem: item ]
		ifFalse: [ AltBrowser roots remove: item ].
	AltBrowser announcer announce: (AltNodeRemoved node: item)! !

!ABRemovePackageCategoryCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/31/2012 15:26'!
icon
	^ #smallCutIcon! !

!ABRemovePackageCategoryCommand methodsFor: 'testing' stamp: 'ThierryGoubier 7/31/2012 15:27'!
isActive
	^ true! !

!ABRemovePackageCategoryCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/17/2012 22:02'!
keystroke
	^ Character delete asShortcut! !

!ABRemovePackageCategoryCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/9/2012 23:39'!
label
	^ 'Remove'! !

ABCommand subclass: #ABRemoveProtocolCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABRemoveProtocolCommand commentStamp: '<historical>' prior: 0!
This command removes a protocol!

!ABRemoveProtocolCommand methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 8/30/2012 16:17'!
execute
	| tClass tItem |
	
	tClass := requestor selectedItem sourceClass.
	tItem := requestor selectedItem.
	tClass removeCategory: tItem item item! !

!ABRemoveProtocolCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/18/2012 14:47'!
icon
	^ #smallDeleteIcon! !

!ABRemoveProtocolCommand methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 8/30/2012 16:13'!
isActive
	^ true! !

!ABRemoveProtocolCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/17/2012 22:01'!
keystroke
	^ Character delete asShortcut! !

!ABRemoveProtocolCommand methodsFor: 'as yet unclassified' stamp: 'ThierryGoubier 9/17/2012 22:03'!
label
	^ 'Remove'! !

ABCommand subclass: #ABSendersCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABSendersCommand methodsFor: 'execution' stamp: 'ThierryGoubier 9/8/2012 11:03'!
execute
	"Execute an all senders of."
	
	SystemNavigation default browseAllCallsOn: requestor selectedItem item item! !

!ABSendersCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/10/2012 22:11'!
icon
	^ #smallFindIcon! !

!ABSendersCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/8/2012 11:03'!
isActive
	^ true! !

!ABSendersCommand methodsFor: 'testing' stamp: 'ThierryGoubier 9/8/2012 11:03'!
isEnabled
	^ true! !

!ABSendersCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 9/8/2012 11:01'!
label
	^ 'Senders'! !

ABCommand subclass: #ABUndoTextCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Alt-Browser-Commands'!

!ABUndoTextCommand commentStamp: '<historical>' prior: 0!
An ABUndoTextCommand calls the text editor undo command.!

!ABUndoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesNodes
	^ false! !

!ABUndoTextCommand class methodsFor: 'testing' stamp: 'lr 12/10/2010 09:25'!
takesText
	^ true! !

!ABUndoTextCommand methodsFor: 'execution' stamp: 'ThierryGoubier 7/26/2012 08:07'!
execute
	target widget textMorph editor undo! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 10:38'!
group
	^ #adapting! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 7/17/2012 23:19'!
icon
	^ #smallUndoIcon! !

!ABUndoTextCommand methodsFor: 'testing' stamp: 'lr 12/10/2010 09:22'!
isActive
	^ true! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'ThierryGoubier 8/1/2012 16:54'!
keystroke
	^ $z ctrl! !

!ABUndoTextCommand methodsFor: 'accessing' stamp: 'lr 12/10/2010 09:20'!
label
	^ 'Undo'! !

!PluggableTextMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/26/2012 07:48'!
trueSelection
	"Return the selection text as remembered by the PluggableTextMoprh, even if not in focus."

	^ textMorph editor text copyFrom: selectionInterval first to: selectionInterval last! !

!PluggableTextMorph methodsFor: '*Alt-Browser' stamp: 'ThierryGoubier 7/25/2012 23:58'!
trueSelectionInterval
	^ selectionInterval! !
AltBrowser initialize!
